# -*- coding: utf-8 -*-
"""Helper for command line functions."""
# This is the same class as: https://github.com/log2timeline/l2tdevtools/ \
# blob/master/l2tdevtools/review_helpers/cli.py
# TODO: Import the review helper directly instead of copy here once projects
# are merged.
import codecs
import locale
import logging
import shlex
import subprocess


class CLIHelper:
  """Command line interface (CLI) helper.

  Attributes:
    mock_responses (dict[str, str]): mappings of commands to responses.
    preferred_encoding (str): preferred encoding of output.
  """

  def __init__(self, mock_responses=None):
    """Initializes a CLI helper.

    Args:
      mock_responses (Optional[dict[str, str]]): mappings of commands to
          responses, for testing.
    """
    super(CLIHelper, self).__init__()
    self.mock_responses = mock_responses
    self.preferred_encoding = locale.getpreferredencoding()

  def RunCommand(self, command: str):
    """Runs a command.

    Args:
      command (str): command to run.

    Returns:
      tuple[int, str, str]: exit code, output that was written to stdout
          and stderr.

    Raises:
      AttributeError: if the command is not recognized.
    """
    if self.mock_responses:
      return_values = self.mock_responses.get(command, None)
      if not return_values:
        raise AttributeError('Unrecognized command.')
      return return_values

    arguments = shlex.split(command)

    try:
      process = subprocess.Popen(
          arguments, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
    except OSError as exception:
      logging.error(
          'Running: "{0:s}" failed with error: {1!s}'.format(
              command, exception))
      return 1, None, None

    output, error = process.communicate()
    output = codecs.decode(output, self.preferred_encoding)
    error = codecs.decode(error, self.preferred_encoding)
    if process.returncode != 0:
      logging.error(
          'Running: "{0:s}" failed with error: {1!s}.'.format(command, error))

    return process.returncode, output, error



# -*- coding: utf-8 -*-
"""Timesketch sketch analyzer scaffolder."""

from l2tscaffolder.scaffolders import timesketch
from l2tscaffolder.scaffolders import manager


class TimesketchSketchScaffolder(timesketch.TimesketchBaseScaffolder):
  """The Timesketch sketch analyzer plugin scaffolder."""

  # The name of the plugin or parser this scaffolder provides.
  NAME = 'sketch_analyzer'
  DESCRIPTION = (
      'Provides a scaffolder to generate a Timesketch sketch analyzer plugin.')

  # Filenames of templates.
  TEMPLATE_PLUGIN_FILE = 'ts_sketch_analyzer.jinja2'
  TEMPLATE_PLUGIN_TEST = 'ts_sketch_analyzer_test.jinja2'


manager.ScaffolderManager.RegisterScaffolder(TimesketchSketchScaffolder)



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the mapping helper."""
import unittest

from l2tscaffolder.lib import mapping_helper
from tests.test_helper import path_helper

# pylint: disable=protected-access
class MappingHelperTest(unittest.TestCase):
  """Tests for the mapping helper."""

  def setUp(self):
    self.template_path = path_helper.TestTemplatePath()
    yapf_path = path_helper.YapfStyleFilePath()
    self.plugin_name = 'the_one_and_only'
    self.file = 'test_template.jinja2'
    self.helper = mapping_helper.MappingHelper(
        formatter_path=yapf_path, template_path=self.template_path)

  def testRender(self):
    """Tests the render method."""
    context = {'plugin_name': self.plugin_name}
    actual = self.helper.RenderTemplate(self.file, context)
    expected = '# -*- coding: utf-8 -*-\n"""{0}"""\n'.format(
        self.plugin_name)
    self.assertEqual(expected, actual)

  def testGenerateClassName(self):
    """Tests the generation of the class name."""
    name = 'this_is_a_test'
    expected = 'ThisIsATest'
    actual = self.helper.GenerateClassName(name)
    self.assertEqual(expected, actual)

  def testRemoveEscapeErrorWithSpacesLikeInTemplate(self):
    """Tests the removing of an escape error"""
    string_with_error = 'u\'the first line\\\'\n        u\'\\the second line'
    string_without_error = 'u\'the first line\'\n        u\'\\\\the second line'
    actual = self.helper._RemoveEscapeError(string_with_error)
    self.assertEqual(string_without_error, actual)

  def testRemoveEscapeErrorWithNoEscape(self):
    """Tests the removing of an escape error"""
    string_without_error = 'u\'the first line\\\\\'\n        u\'the second line'
    actual = self.helper._RemoveEscapeError(string_without_error)
    self.assertEqual(string_without_error, actual)

  def testRemovewhitespaceAtEndOfLineForTwowhitespace(self):
    """Tests the removing of whitespace at the end of a line."""
    string_with_blank = 'somestuff  \n'
    string_without_blank = 'somestuff\n'
    actual = self.helper._RemoveWhitespaceAtEndOfLine(string_with_blank)
    self.assertEqual(string_without_blank, actual)

  def testRemoveWhitespaceAtEndOfLineForFourWhitespace(self):
    """Tests the removing of whitespace at the end of a line."""
    string_with_blank = '    \n'
    string_without_blank = '\n'
    actual = self.helper._RemoveWhitespaceAtEndOfLine(string_with_blank)
    self.assertEqual(string_without_blank, actual)

  def testRemoveWhitespaceAtEndOfLineFor12Whitespace(self):
    """Tests the removing of whitespace at the end of a line"""
    string_with_blank = 'somestuff            \n'
    string_without_blank = 'somestuff\n'
    actual = self.helper._RemoveWhitespaceAtEndOfLine(string_with_blank)
    self.assertEqual(string_without_blank, actual)

  def testRemoveYapfCommentOnlyTheEnable(self):
    """Tests the removing of the yapf enable comment"""
    string_with_yapf_enable = 'something\n# yapf: enable\notherline'
    string_without_yapf_enable = 'something\notherline'
    actual = self.helper._RemoveYapfComment(string_with_yapf_enable)
    self.assertEqual(string_without_yapf_enable, actual)

  def testRemoveYapfCommentOnlyTheDisable(self):
    """Tests the removing of the yapf enable comment"""
    string_with_yapf_enable = 'something\n# yapf: disable\notherline'
    string_without_yapf_enable = 'something\notherline'
    actual = self.helper._RemoveYapfComment(string_with_yapf_enable)
    self.assertEqual(string_without_yapf_enable, actual)

  def testRemoveYapfCommentOnlyTheEnableAndDisable(self):
    """Tests the removing of the yapf enable comment"""
    string_with_yapf_enable = ('something\n'
                               '# yapf: disable\n'
                               'otherline\n'
                               '# yapf: enable\n'
                               'lastline')
    string_without_yapf_enable = 'something\notherline\nlastline'
    actual = self.helper._RemoveYapfComment(string_with_yapf_enable)
    self.assertEqual(string_without_yapf_enable, actual)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file imports Python modules that registers scaffolders."""
from l2tscaffolder.scaffolders import plaso_sqlite
from l2tscaffolder.scaffolders import timesketch_index
from l2tscaffolder.scaffolders import timesketch_sketch
from l2tscaffolder.scaffolders import turbinia_job_task



# -*- coding: utf-8 -*-
"""Formatter for generated code."""

from yapf.yapflib import yapf_api


class CodeFormatter:
  """Formats code in files."""

  def __init__(self, yapf_path: str):
    """Initializes the code formatter.

    Args:
      yapf_path (str): path to the yapf style file.
    """
    super().__init__()
    self.yapf_path = yapf_path

  def Format(self, code: str) -> str:
    """Formats the code.

    Args:
      code (str): code to format

    Returns:
      str: the formatted code
    """
    return yapf_api.FormatCode(code, style_config=self.yapf_path)



# -*- coding: utf-8 -*-
"""This file imports Python modules that registers definitions."""

from l2tscaffolder.definitions import plaso
from l2tscaffolder.definitions import timesketch
from l2tscaffolder.definitions import turbinia



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Timesketch sketch analyzer scaffolder."""
import unittest

from l2tscaffolder.scaffolders import timesketch_sketch


class TimesketchSketchScaffolderTest(unittest.TestCase):
  """Test class for the Timesketch sketch analyzer plugin scaffolder."""

  maxDiff = None

  def testTimesketchSketchScaffolder(self):
    """Test the Timesketch index analyzer scaffolder."""
    scaffolder = timesketch_sketch.TimesketchSketchScaffolder()
    scaffolder.SetOutputName('testing')

    file_copy_paths = [x for _, x in scaffolder.GetFilesToCopy()]
    self.assertEqual(file_copy_paths, [])

    files_generated = dict(scaffolder.GenerateFiles())

    expected_files = frozenset([
        'timesketch/lib/analyzers/testing.py',
        'timesketch/lib/analyzers/testing_test.py'])
    self.assertEqual(set(files_generated.keys()), expected_files)

    expected_init_files = frozenset([
        'timesketch/lib/analyzers/__init__.py'])
    init_generated = dict(scaffolder.GetInitFileChanges())
    self.assertEqual(set(init_generated.keys()), expected_init_files)

    expected_parser_init_addition = (
        'from timesketch.lib.analyzers import testing\n')
    self.assertEqual(
        expected_parser_init_addition,
        init_generated['timesketch/lib/analyzers/__init__.py'])

    with open('test_data/timesketch_testing_plugin.py', 'r') as fh:
      expected_parser_content = fh.read()
    self.assertEqual(
        expected_parser_content,
        files_generated['timesketch/lib/analyzers/testing_test.py'])


if __name__ == '__main__':
  unittest.main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the plaso sqlite scaffolder."""
import unittest

from l2tscaffolder.lib import errors
from l2tscaffolder.scaffolders import plaso_sqlite


class PlasoSQLiteScaffolderTest(unittest.TestCase):
  """Test class for the plaso sqlite scaffolder."""

  maxDiff = None

  def _RunQueryTests(self, scaffolder, test_string, expected_columns):
    """Test query columns function.

    Args:
      scaffolder (plaso_sqlite.PlasoSQLiteScaffolder): scaffolder to test.
      test_string (str): string to test the _GetQueryColumns method on.
      expected_columns (set[str]): columns names expected to be extracted from
          the test string.
    """
    # pylint: disable=protected-access
    columns = set(list(scaffolder._GetQueryColumns(test_string)))
    self.assertEqual(columns, expected_columns)

  def testQueryColumns(self):
    """Test query columns function."""
    scaffolder = plaso_sqlite.PlasoSQLiteScaffolder()
    test_string = (
        'SELECT foobar as Foo, foobar.dot, random, reallylong AS long FROM '
        'foobarengine WHERE foobar = 1')
    expected_columns = set(['foo', 'dot', 'random', 'long'])
    self._RunQueryTests(scaffolder, test_string, expected_columns)

    test_string = (
        'select one, two as three, four as five, f.eight as EIGHTE FROM '
        'foobar f, scode s WHERE f.id = s.id ORDER BY one')
    expected_columns = set(['one', 'three', 'five', 'eighte'])
    self._RunQueryTests(scaffolder, test_string, expected_columns)

    test_string = (
        'this should not produce anything...')
    self._RunQueryTests(scaffolder, test_string, set())

  def testPlasoSQLiteScaffolder(self):
    """Test the plaso SQLite scaffolder."""
    scaffolder = plaso_sqlite.PlasoSQLiteScaffolder()

    scaffolder.SetOutputName('testing')

    with self.assertRaises(errors.ScaffolderNotConfigured):
      scaffolder.RaiseIfNotReady()

    queries = {
        'strange': 'SELECT name, address, ssn FROM strange',
        'foobar': (
            'SELECT f1.foo, f2.bar AS Bar FROM foobar_one AS f1, '
            'foobar_two as f2 WHERE f1.id = f2.id')}
    required_tables = ['foobar_one', 'foobar_two', 'strange_table']
    scaffolder.SetAttribute('queries', queries, dict)
    scaffolder.SetAttribute('required_tables', required_tables, list)
    scaffolder.SetAttribute('test_file', 'test_data/test_sqlite.db', str)

    file_copy_paths = [x for _, x in scaffolder.GetFilesToCopy()]
    self.assertEqual(file_copy_paths, ['test_data/test_sqlite.db'])

    files_generated = dict(scaffolder.GenerateFiles())

    expected_files = frozenset([
        'plaso/formatters/testing.py', 'tests/formatters/testing.py',
        'plaso/parsers/sqlite_plugins/testing.py',
        'tests/parsers/sqlite_plugins/testing.py'])
    self.assertEqual(set(files_generated.keys()), expected_files)

    expected_init_files = frozenset([
        'plaso/formatters/__init__.py',
        'plaso/parsers/sqlite_plugins/__init__.py'])
    init_generated = dict(scaffolder.GetInitFileChanges())
    self.assertEqual(set(init_generated.keys()), expected_init_files)

    expected_parser_init_addition = (
        'from '
        'plaso.parsers.sqlite_plugins import testing\n')
    self.assertEqual(
        expected_parser_init_addition,
        init_generated['plaso/parsers/sqlite_plugins/__init__.py'])

    with open('test_data/plaso_testing_sqlite_plugin.py', 'r') as fh:
      expected_parser_content = fh.read()
    self.assertEqual(
        expected_parser_content,
        files_generated['plaso/parsers/sqlite_plugins/testing.py'])


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Interface defining how a project class looks like."""
import abc

from l2tscaffolder.lib import definitions

class ScaffolderDefinition:
  """Scaffolder definition interface."""

  NAME = definitions.DEFINITION_UNDEFINED

  @abc.abstractmethod
  def ValidatePath(self, root_path: str) -> bool:
    """Validates the path to the root directory of the project.

    Args:
      root_path (str): the path to the root of the project directory.

    Returns:
      bool: whether the given path is the correct root path of the project.
    """



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the code formatter."""
import unittest

from l2tscaffolder.lib import code_formatter
from tests.test_helper import path_helper


class CodeFormatterTest(unittest.TestCase):
  """Test case for the code formatter functions. """

  def testCodeFormatter(self):
    """Tests the code formatter.."""
    yapf_path = path_helper.YapfStyleFilePath()
    formatter = code_formatter.CodeFormatter(yapf_path)

    faulty_code_string = (
        'class Foobar(object):\n'
        '\n'
        '  def SetStuff(self):\n'
        '  return None\n')

    with self.assertRaises(IndentationError):
      _ = formatter.Format(faulty_code_string)

    code_string = (
        'class Foobar(object):\n  """This is a class like no other '
        'class."""\n\n  def Foobar(self, s):\n    """Foobar is the foobar."""\n'
        '\n    thisIsAReallyLongVariable = os.path.join('
        '"foobar", "another", "lets see", "yet", "formal", "informal", "direct"'
        ', "indirect")\n')
    formatted_code, code_changed = formatter.Format(code_string)

    correct_code = (
        'class Foobar(object):\n'
        '  """This is a class like no other class."""\n'
        '\n'
        '  def Foobar(self, s):\n'
        '    """Foobar is the foobar."""\n'
        '\n'
        '    thisIsAReallyLongVariable = os.path.join(\n'
        '        "foobar", "another", "lets see", "yet", "formal", '
        '"informal", "direct",\n'
        '        "indirect")\n')

    self.assertTrue(code_changed)
    self.assertEqual(formatted_code, correct_code)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The scaffolder manager."""

from typing import Dict
from typing import Iterator
from typing import List
from typing import Optional
from typing import Tuple
from typing import Type

from l2tscaffolder.scaffolders import interface


class ScaffolderManager:
  """The scaffolder manager."""

  _scaffolder_classes = {}

  @classmethod
  def DeregisterScaffolder(cls, scaffolder_class: Type[interface.Scaffolder]):
    """Deregisters a scaffolder class.

    The scaffolder classes are identified based on their lower case name.

    Args:
      scaffolder_class (type): scaffolder class (subclass of Scaffolder).

    Raises:
      KeyError: if scaffolder class is not set for the corresponding name.
    """
    scaffolder_name = scaffolder_class.NAME.lower()
    if scaffolder_name not in cls._scaffolder_classes:
      raise KeyError('Scaffolder class not set for name: {0:s}.'.format(
          scaffolder_class.NAME))

    del cls._scaffolder_classes[scaffolder_name]

  @classmethod
  def GetScaffolderClasses(cls) -> Iterator[Type[interface.Scaffolder]]:
    """Generates a list of all registered scaffolder classes."""
    for scaffolder_class in cls._scaffolder_classes.values():
      yield scaffolder_class

  @classmethod
  def GetScaffolderNames(cls) -> Iterator[str]:
    """Retrieves the scaffolder names.

    Yields:
      str: scaffolder names.
    """
    for scaffolder_name, _ in cls.GetScaffolders():
      yield scaffolder_name

  @classmethod
  def GetScaffolderInformation(cls) -> Iterator[Tuple[str, str]]:
    """Retrieves the scaffolder information.

    Yields:
      tuple[str, str]: pairs of scaffolder names and descriptions.
    """
    for scaffolder_name, scaffolder_class in cls.GetScaffolders():
      yield (scaffolder_name, scaffolder_class.DESCRIPTION)

  @classmethod
  def GetScaffolderObjectByName(
      cls, scaffolder_name) -> Optional[interface.Scaffolder]:
    """Retrieves a specific scaffolder object by its name.

    Args:
      scaffolder_name (str): name of the scaffolder.

    Returns:
      Scaffolder: scaffolder object or None.
    """
    scaffolder_class = cls._scaffolder_classes.get(
        scaffolder_name.lower(), None)
    if scaffolder_class:
      return scaffolder_class()
    return None

  @classmethod
  def GetScaffolderObjects(cls) -> Dict[str, interface.Scaffolder]:
    """Retrieves the scaffolder objects.

    Returns:
      dict[str, Scaffolder]: scaffolders per name.
    """
    scaffolder_objects = {}
    for scaffolder_name, scaffolder_class in cls._scaffolder_classes.items():
      scaffolder_object = scaffolder_class()
      scaffolder_objects[scaffolder_name] = scaffolder_object

    return scaffolder_objects

  @classmethod
  def GetScaffolderQuestions(cls) -> List[interface.BaseQuestion]:
    """Retrieves all the questions asked by scaffolders.

    Returns:
      list[interface.BaseQuestion]: questions asked by all scaffolders.
    """
    questions = []
    for scaffolder_class in cls._scaffolder_classes.values():
      scaffolder_object = scaffolder_class()
      questions.extend(scaffolder_object.GetQuestions())

    return questions

  @classmethod
  def GetScaffolderQuestionByName(
      cls, scaffolder_name: str) -> List[interface.BaseQuestion]:
    """Retrieve a list of questions asked by a scaffolder based on name.

    Args:
      scaffolder_name (str): name of the scaffolder.

    Returns:
      list: a list with all the questions needed to setup the  scaffolder.
          If scaffolder_name is not registered an empty list will be returned.
    """
    scaffolder_class = cls._scaffolder_classes.get(
        scaffolder_name.lower(), None)
    if not scaffolder_class:
      return list()

    scaffolder_object = scaffolder_class()
    return scaffolder_object.GetQuestions()

  @classmethod
  def GetScaffolders(cls) -> Iterator[Tuple[str, Type[interface.Scaffolder]]]:
    """Retrieves the registered scaffolders.

    Retrieves a dictionary of all registered scaffolders.

    Yields:
      tuple: contains:

      * str: name of the scaffolder:
      * type: scaffolder class (subclass of Scaffolder).
    """
    for scaffolder_name, scaffolder_class in cls._scaffolder_classes.items():
      yield scaffolder_name, scaffolder_class

  @classmethod
  def RegisterScaffolder(cls, scaffolder_class: Type[interface.Scaffolder]):
    """Registers a scaffolder class.

    The scaffolder classes are identified based on their lower case name.

    Args:
      scaffolder_class (type): scaffolder class (subclass of Scaffolder).

    Raises:
      KeyError: if scaffolder class is already set for the corresponding name.
    """
    scaffolder_name = scaffolder_class.NAME.lower()
    if scaffolder_name in cls._scaffolder_classes:
      raise KeyError('Scaffolder class already set for name: {0:s}.'.format(
          scaffolder_class.NAME))

    cls._scaffolder_classes[scaffolder_name] = scaffolder_class

  @classmethod
  def RegisterScaffolders(
      cls, scaffolder_classes: List[Type[interface.Scaffolder]]):
    """Registers scaffolder classes.

    The scaffolder classes are identified based on their lower case name.

    Args:
      scaffolder_classes (list[type]): scaffolders classes
          (subclasses of Scaffolder).

    Raises:
      KeyError: if scaffolder class is already set for the corresponding name.
    """
    for scaffolder_class in scaffolder_classes:
      cls.RegisterScaffolder(scaffolder_class)



# -*- coding: utf-8 -*-
"""The definition manager."""

from typing import Iterator
from typing import Type

from l2tscaffolder.definitions import interface


class DefinitionManager:
  """The definition manager."""

  _definition_classes = {}

  @classmethod
  def DeregisterDefinition(
      cls, definition_class: Type[interface.ScaffolderDefinition]):
    """Deregisters a definition class.

    Definition classes are identified by their NAME attribute.

    Args:
      definition_class (type): definition class (subclass of
          ScaffolderDefinition).

    Raises:
      KeyError: if definition class is not set for the corresponding name.
    """
    definition_name = definition_class.NAME

    if definition_name not in cls._definition_classes:
      raise KeyError('Definition class not set for name: {0:s}.'.format(
          definition_name))

    del cls._definition_classes[definition_name]

  @classmethod
  def GetDefinitionByName(
      cls, name: str) -> Type[interface.ScaffolderDefinition]:
    """Returns a definition class based on registered name.

    Args:
      name (str): name of the definition.

    Returns:
      interface.ScaffolderDefinition: definition class or None
          if name is not registered.
    """
    return cls._definition_classes.get(name, None)

  @classmethod
  def GetDefinitionNames(cls) -> Iterator[str]:
    """Yields all names of registered definition classes.

    Yields:
      str: definition names.
    """
    for definition_name in cls._definition_classes:
      yield definition_name

  @classmethod
  def GetDefinitionObjects(cls) -> Iterator[interface.ScaffolderDefinition]:
    """Yields instances of each registered definition class.

    Yields:
      ScaffolderDefinition: definition object.
    """
    for definition_class in cls._definition_classes.values():
      yield definition_class()

  @classmethod
  def RegisterDefinition(
      cls, definition_class: Type[interface.ScaffolderDefinition]):
    """Registers a definition class.

    Definition classes are identified by their NAME attribute.

    Args:
      definition_class (ScaffolderDefinition): definition class.

    Raises:
      KeyError: if definition class is already set for the corresponding name.
    """
    definition_name = definition_class.NAME

    if definition_name in cls._definition_classes:
      raise KeyError('Project class already set for name: {0:s}.'.format(
          definition_name))

    cls._definition_classes[definition_name] = definition_class



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""The scaffolder frontend."""
import os
from typing import Dict
from typing import List

from l2tscaffolder.frontend import output_handler as handler
from l2tscaffolder.helpers import git
from l2tscaffolder.lib import engine
from l2tscaffolder.lib import errors

from l2tscaffolder.definitions import interface as definition_interface
from l2tscaffolder.definitions import manager as definition_manager
from l2tscaffolder.scaffolders import interface as scaffolder_interface
from l2tscaffolder.scaffolders import manager as scaffolder_manager


class ScaffolderFrontend:
  """A frontend implementation for the scaffolder project."""

  def __init__(self, output_handler: handler.BaseOutputHandler):
    """Initializes the frontend.

    Args:
      output_handler (handler.BaseOutputHandler): the output handler used
          for the frontend.
    """
    self._git_helper = None
    self._output_handler = output_handler

  def _AskDictQuestion(
      self, question: scaffolder_interface.DictQuestion) -> Dict[str, str]:
    """Prompts the user with a question and return a dict answer back.

    The user is prompted to supply both the key and value for each
    entry in the dict. The key is the first response from the user and
    value the second one. If the user attempts to add an entry with a key
    that already exists an error is presented to the user and they are asked
    to try again.

    Args:
      question (scaffolder_interface.DictQuestion): the question to ask.

    Returns:
      dict[str, str]: keys and values provided by the user.
    """
    return_dict = {}

    more_entries = True
    entry_index = 1
    while more_entries:
      key = self._output_handler.PromptInfo(
          '{0:s} [#{1:d}]'.format(question.key_prompt, entry_index))
      if key in return_dict:
        self._output_handler.PrintError(
            'Key value ({0:s}) already entered.'.format(key))
        continue

      value = self._output_handler.PromptInfo(
          '{0:s} [#{1:d}]'.format(question.value_prompt, entry_index))

      return_dict[key] = value
      more_entries = self._output_handler.Confirm('More entries?', abort=False)
      entry_index += 1

    question.ValidateAnswer(return_dict)
    return return_dict

  def _AskListQuestion(
      self, question: scaffolder_interface.ListQuestion) -> List[str]:
    """Prompts the user with a question and return a list back.

    Args:
      question (scaffolder_interface.ListQuestion): the question to ask.

    Returns:
      list: the answer as supplied by the user.
    """
    return_list = []

    more_entries = True
    entry_index = 1
    while more_entries:
      value = self._output_handler.PromptInfo(
          'Value to add [#{0:d}]'.format(entry_index))
      if not value:
        self._output_handler.PrintError('Empty value, not adding.')
      else:
        return_list.append(value)

      more_entries = self._output_handler.Confirm(
          'Add more values?', abort=False)
      entry_index += 1

    question.ValidateAnswer(return_list)
    return return_list

  def _AskQuestion(self, question: scaffolder_interface.BaseQuestion) -> object:
    """Prompts the user with a question and return an answer back.

    Args:
      question (scaffolder_interface.BaseQuestion): the question to ask.

    Returns:
      object: an object, whose type is defined by the question.

    Raises:
      errors.WrongCliInput: when an unsupported question type is
          encountered.
    """
    self._output_handler.PrintNewLine()
    self._output_handler.PrintInfo(question.prompt)

    if isinstance(question, scaffolder_interface.StringQuestion):
      return self._AskStringQuestion(question)

    if isinstance(question, scaffolder_interface.DictQuestion):
      return self._AskDictQuestion(question)

    if isinstance(question, scaffolder_interface.ListQuestion):
      return self._AskListQuestion(question)

    # TODO: Add support for other types of questions.
    raise errors.WrongCliInput(
        'Question type {0:s} not supported.'.format(str(question.__class__)))

  def _AskStringQuestion(
      self, question: scaffolder_interface.StringQuestion) -> str:
    """Prompts the user with a question and return a string answer back.

    Args:
      question (scaffolder_interface.StringQuestion): the question to ask.

    Returns:
      str: the answer as supplied by the user.
    """
    value = self._output_handler.PromptInfo('Value')
    question.ValidateAnswer(value)
    return value

  def _GetSelection(self, items: list, item_text: str) -> str:
    """Presents a list of strings to user and return back user choice.

    Args:
      items (list): list of strings to present to the user.
      item_text (str): heading used in presentation to the user.

    Returns:
      str: the chosen string from the list.

    Raises:
      KeyError: if the user selection is not part of the list or
          it's not a valid number.
    """
    for item_count, item in enumerate(items):
      self._output_handler.PrintInfo(
          '  [{0:d}] {1:s}'.format(item_count, item))

    result = self._output_handler.PromptInfo('{0:s} choice'.format(item_text))
    try:
      result_int = int(result, 10)

      if 0 <= result_int < len(items):
        return items[result_int]
    except ValueError:
      raise KeyError('Unable to convert {0:s} into a number.'.format(result))

    raise KeyError(
        'Item [{0:d}] not in list, please pick a valid number.'.format(
            result_int))

  def CreateGitFeatureBranch(self, project_path: str, module_name: str):
    """Creates a feature branch inside the git project.

    Creates a feature branch inside the git project path
    to store all the generated files in.

    Args:
      project_path (str): path to the git project folder.
      module_name (str): name of the output module.
    """
    branch_name = self._git_helper.GenerateBranchName(module_name)
    if not self._git_helper.HasBranch(branch_name):
      self._output_handler.PrintOutput(
          'Creating feature branch: {0:s} inside {1:s}'.format(
              branch_name, project_path))
      self._git_helper.CreateBranch(branch_name)

    self._output_handler.PrintOutput('Switching to feature branch {0:s}'.format(
        branch_name))
    self._git_helper.SwitchToBranch(branch_name)

  def GatherScaffolderAnswers(self, scaffolder, scaffolder_engine):
    """Asks all questions that scaffolder requires and store the results in it.

    Args:
      scaffolder (scaffolder_interface.Scaffolder): the scaffolder that
          stores all required questions and stores all results as well.
      scaffolder_engine (scaffolder_engine.ScaffolderEngine): the scaffolder
          engine object, needed to store answers from questions asked.

    Raises:
      UnableToConfigure: if the answer causes the scaffolder not
          to be configured properly.
    """
    for question in scaffolder.GetQuestions():
      gather_answer = True
      # Loop created to give the user a chance to correct each incorrect
      # answer they provide.
      while gather_answer:
        try:
          value = self._AskQuestion(question)
          scaffolder_engine.StoreScaffolderAttribute(
              question.attribute, value, question.TYPE)
          break
        except errors.UnableToConfigure as exception:
          self._output_handler.PrintError(
              'Unable to configure, with error: {0:s}'.format(repr(exception)))
          gather_answer = self._output_handler.Confirm('Want to try again?')
          if not gather_answer:
            raise

  def GetDefinition(
      self,
      definition_string: str) -> definition_interface.ScaffolderDefinition:
    """Returns the definition object as chosen by the user.

    Args:
      definition_string (str): definition name, read from user input.

    Returns:
      definition_interface.ScaffolderDefinition: the chosen definition object.
    """
    definitions = list(
        definition_manager.DefinitionManager.GetDefinitionNames())

    self._output_handler.PrintNewLine()
    if not definition_string:
      self._output_handler.PrintInfo('Available definitions: ')
      definition_string = ''
      while not definition_string:
        try:
          definition_string = self._GetSelection(definitions, 'Definition')
        except KeyError as e:
          self._output_handler.PrintError('{0!s}'.format(e))

    if definition_string in definitions:
      self._output_handler.PrintOutput(
          '{0:s} chosen.'.format(definition_string))
      def_class = definition_manager.DefinitionManager.GetDefinitionByName(
          definition_string)
      return def_class()

    self._output_handler.PrintError(
        'Definition {0:s} does not exist.'.format(definition_string))
    return self.GetDefinition('')

  def GetModuleName(self) -> str:
    """Returns the module name as chosen by the user."""
    self._output_handler.PrintNewLine()
    self._output_handler.PrintInfo((
        'Name of the module to be generated. This can be something like "'
        'foobar sqlite" or "event analytics".\n\nThis will be used for class '
        'name generation and file name prefixes.'))
    return self._output_handler.PromptInfo('Module Name')

  def GetProjectPath(
      self, definition: definition_interface.ScaffolderDefinition) -> str:
    """Returns the path to the project's root folder as chosen by the user.

    Args:
      definition (definition_interface.ScaffolderDefinition): the chosen
          definition. Used to validate the project path.

    Returns:
      str: the path to the project's root folder.

    Raises:
      errors.WrongCliInput: when no valid project path has been provided.
    """
    self._output_handler.PrintNewLine()
    project_path = self._output_handler.PromptInfo('Path to the project root')
    if definition.ValidatePath(project_path):
      self._output_handler.PrintOutput(
          'Path [{0:s}] set as the project path.'.format(project_path))
      self._git_helper = git.GitHelper(project_path)
      return project_path

    check = self._output_handler.Confirm((
        'Path [{0:s}] does not lead to a valid project for {1:s}. '
        'Do you want to try again?').format(project_path, definition.NAME))

    if check:
      return self.GetProjectPath(definition)

    raise errors.WrongCliInput(
        'Unable to proceed without a valid project path.')

  def GetScaffolder(
      self, definition: definition_interface.ScaffolderDefinition
  ) -> scaffolder_interface.Scaffolder:
    """Returns the scaffolder as chosen by the user.

    Args:
      definition (definition_interface.ScaffolderDefinition): the chosen
          definition. Used to determine available scaffolders.

    Returns:
      scaffolder_interface.ScaffolderCli: the chosen scaffolder object.
    """
    scaffolders = {}
    manager = scaffolder_manager.ScaffolderManager
    for scaffolder_name, scaffolder in manager.GetScaffolders():
      if scaffolder.PROJECT == definition.NAME:
        scaffolders[scaffolder_name] = scaffolder

    self._output_handler.PrintNewLine()
    self._output_handler.PrintInfo(
        'Available scaffolders for {0:s}:'.format(definition.NAME))

    scaffolder = ''
    while not scaffolder:
      try:
        scaffolder = self._GetSelection(list(scaffolders.keys()), 'Scaffolder')
      except KeyError as exception:
        self._output_handler.PrintError('{0!s}'.format(exception))
    if scaffolder in scaffolders:
      return scaffolders[scaffolder]()

    self._output_handler.PrintError(
        'Scaffolder: {0:s} does not exist.'.format(scaffolder))
    return self.GetScaffolder(definition)

  def Start(self, definition_value):
    """Start the CLI.

    Args:
      definition_value (str): the definition string chosen by UI.
    """
    self._output_handler.PrintInfo('   == Starting the scaffolder ==')
    self._output_handler.PrintInfo('Gathering required information.')
    scaffolder_engine = engine.ScaffolderEngine()

    definition = self.GetDefinition(definition_value)

    project_path = self.GetProjectPath(definition)
    scaffolder_engine.SetProjectRootPath(project_path)

    module_name = self.GetModuleName()
    scaffolder_engine.SetModuleName(module_name)
    self._output_handler.PrintInfo(
        'About to create a new feature branch to store newly generated code.')
    try:
      self.CreateGitFeatureBranch(project_path, scaffolder_engine.module_name)
    except errors.UnableToConfigure as exception:
      self._output_handler.PrintError((
          'Unable to create feature branch, is {0:s} a valid git project path? '
          'The error message was: {1!s}').format(project_path, exception))
      self._output_handler.PrintError('Due to fatal error, not proceeding.')
      return

    scaffolder = self.GetScaffolder(definition)
    scaffolder_engine.SetScaffolder(scaffolder)
    try:
      self.GatherScaffolderAnswers(scaffolder, scaffolder_engine)
    except errors.UnableToConfigure as exception:
      self._output_handler.PrintError(
          ('Aborting. Unable to properly configure scaffolder '
           'with error {0!s}.').format(exception))
      return

    ready = self._output_handler.Confirm('Ready to generate files?')
    if ready:
      for file_path in scaffolder_engine.GenerateFiles():
        self._output_handler.PrintOutput(
            'File: {0:s} written to disk.'.format(file_path))
        _, _, file_path_inside_project = file_path.partition(project_path)
        if file_path_inside_project.startswith(os.sep):
          file_path_inside_project = file_path_inside_project[1:]
        self._git_helper.AddFileToTrack(file_path_inside_project)



# -*- coding: utf-8 -*-
"""Tests for the Turbinia job and task scaffolder."""
import unittest

from l2tscaffolder.scaffolders import turbinia_job_task


class TurbiniaJobTaskScaffolderTest(unittest.TestCase):
  """Test class for the Turbinia job and task scaffolder."""

  maxDiff = None

  def testTurbiniaJobTaskScaffolder(self):
    """Test the Turbinia job and task scaffolder."""
    scaffolder = turbinia_job_task.TurbiniaJobTaskScaffolder()
    scaffolder.SetOutputName('secret_processing')

    file_copy_paths = [x for _, x in scaffolder.GetFilesToCopy()]
    self.assertEqual(file_copy_paths, [])

    files_generated = dict(scaffolder.GenerateFiles())

    expected_files = frozenset([
        'turbinia/jobs/secret_processing.py',
        'turbinia/workers/secret_processing.py'])
    self.assertEqual(set(files_generated.keys()), expected_files)

    expected_init_files = frozenset([
        'turbinia/jobs/__init__.py'])
    init_generated = dict(scaffolder.GetInitFileChanges())
    self.assertEqual(set(init_generated.keys()), expected_init_files)

    with open('test_data/turbinia_job_output.py', 'r') as fh:
      expected_parser_content = fh.read()
    self.assertEqual(
        expected_parser_content,
        files_generated['turbinia/jobs/secret_processing.py'])


if __name__ == '__main__':
  unittest.main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the scaffolder manager."""
import unittest

from l2tscaffolder.scaffolders import interface
from l2tscaffolder.scaffolders import manager


class BaseScaffolderTest(interface.Scaffolder):
  """Basic class to inherit in tests."""

  def GenerateFiles(self):
    """Empty file generator."""

  def GetFilesToCopy(self):
    """Empty files to copy."""


class AwesomeTestScaffolder(BaseScaffolderTest):
  """First test scaffolder."""
  NAME = 'Awesome'
  DESCRIPTION = 'This is a really awesome thing.'
  QUESTIONS = [
      interface.StringQuestion('test1', 'enter the test'),
      interface.StringQuestion('test2', 'enter the test'),
      interface.StringQuestion('test3', 'enter the test')]


class AverageTestScaffolder(BaseScaffolderTest):
  """Second test scaffolder."""
  NAME = 'Average'
  DESCRIPTION = 'This scaffolder implements the average parser.'
  QUESTIONS = [
      interface.StringQuestion('mediocre', 'a'),
      interface.StringQuestion('lala', 'a'),
      interface.StringQuestion('ok', 'enter')]


class RegistrationTestScaffolder(BaseScaffolderTest):
  """Third test scaffolder."""
  NAME = 'Registration'
  DESCRIPTION = (
      'This scaffolder implements the registration scaffolder, required '
      'paperwork for many ISO standards of the future.')
  QUESTIONS = [
      interface.StringQuestion('27001', 'a'),
      interface.StringQuestion('9001', 'a'),
      interface.StringQuestion('3120512', 'a'),
      interface.ListQuestion('9001-2', 'enter the list here'),
      interface.StringQuestion('12362323', 'a')]


class NotAwesomeTestScaffolder(AwesomeTestScaffolder):
  """Test scaffolder for the Register function."""
  NAME = 'Clearly not awesome'


class ScaffolderManagerTest(unittest.TestCase):
  """Test class for the scaffolder manager."""

  @classmethod
  def setUpClass(cls):
    """Setup tests by registering three scaffolders and removing all others."""
    scaffolders = list(manager.ScaffolderManager.GetScaffolderClasses())
    for scaffolder in scaffolders:
      manager.ScaffolderManager.DeregisterScaffolder(scaffolder)

    manager.ScaffolderManager.RegisterScaffolders(
        [AwesomeTestScaffolder, AverageTestScaffolder,
         RegistrationTestScaffolder])

  def testDeregisterScaffolder(self):
    """Testing whether we can remove a scaffolder from the registration."""
    number_of_scaffolders = len(
        list(manager.ScaffolderManager.GetScaffolders()))
    self.assertEqual(number_of_scaffolders, 3)

    manager.ScaffolderManager.DeregisterScaffolder(AwesomeTestScaffolder)
    number_of_scaffolders = len(
        list(manager.ScaffolderManager.GetScaffolders()))
    self.assertEqual(number_of_scaffolders, 2)

    with self.assertRaises(KeyError):
      manager.ScaffolderManager.DeregisterScaffolder(NotAwesomeTestScaffolder)

    # Let's register it again for other tests to succeed.
    manager.ScaffolderManager.RegisterScaffolder(AwesomeTestScaffolder)

  def testGetScaffolderNames(self):
    """Testing the scaffolder names."""
    scaffolder_names = list(manager.ScaffolderManager.GetScaffolderNames())
    self.assertEqual(len(scaffolder_names), 3)

    correct_names = set(['awesome', 'average', 'registration'])
    scaffolder_name_set = set(scaffolder_names)

    self.assertSetEqual(correct_names, scaffolder_name_set)

  def testGetScaffolderInformation(self):
    """Testing the scaffolder information gathering of the manager."""
    scaffolder_description = {}
    for name, desc in manager.ScaffolderManager.GetScaffolderInformation():
      scaffolder_description[name] = desc

    self.assertEqual(len(scaffolder_description.keys()), 3)

    self.assertEqual(
        scaffolder_description.get('awesome', None),
        'This is a really awesome thing.')

    self.assertEqual(
        scaffolder_description.get('average', None),
        'This scaffolder implements the average parser.')

  def testGetScaffolderObjectByName(self):
    """Testing getting the scaffolder object by name."""
    scaffolder = manager.ScaffolderManager.GetScaffolderObjectByName(
        RegistrationTestScaffolder.NAME)

    self.assertEqual(scaffolder.NAME, RegistrationTestScaffolder.NAME)

  def testGetScaffolderObjects(self):
    """Testing scaffolder objects."""
    scaffolder_objects = manager.ScaffolderManager.GetScaffolderObjects()
    self.assertEqual(len(scaffolder_objects), 3)

    attributes = [
        scaffolder.NAME.lower() for scaffolder in scaffolder_objects.values()]
    correct = ['awesome', 'average', 'registration']

    self.assertSetEqual(set(attributes), set(correct))

  def testGetScaffolderQuestions(self):
    """Test getting all scaffolder questions."""
    questions = manager.ScaffolderManager.GetScaffolderQuestions()
    question_count = len(AwesomeTestScaffolder.QUESTIONS)
    question_count += len(AverageTestScaffolder.QUESTIONS)
    question_count += len(RegistrationTestScaffolder.QUESTIONS)

    self.assertEqual(question_count, len(questions))

    question_attributes = [x.attribute for x in questions]

    self.assertIn('test1', question_attributes)
    self.assertIn('lala', question_attributes)
    self.assertIn('27001', question_attributes)

  def testGetScaffolderQuestionByName(self):
    """Test fetching questions of a scaffolder scaffolder by NAME attribute."""
    questions = manager.ScaffolderManager.GetScaffolderQuestionByName(
        'registration')

    self.assertEqual(len(questions), 5)

    question_attributes = [question.attribute for question in questions]
    self.assertIn('27001', question_attributes)

  def testGetScaffolders(self):
    """Test getting scaffolders from the manager."""
    scaffolders = dict(list(manager.ScaffolderManager.GetScaffolders()))

    self.assertEqual(len(scaffolders.keys()), 3)
    correct = ['awesome', 'average', 'registration']
    self.assertEqual(set(scaffolders.keys()), set(correct))

  def testRegisterScaffolder(self):
    """Test registering new scaffolders."""
    with self.assertRaises(KeyError):
      manager.ScaffolderManager.RegisterScaffolder(AverageTestScaffolder)

    self.assertEqual(
        len(list(manager.ScaffolderManager.GetScaffolderNames())), 3)
    manager.ScaffolderManager.RegisterScaffolder(NotAwesomeTestScaffolder)
    scaffolder_names = list(manager.ScaffolderManager.GetScaffolderNames())
    self.assertEqual(len(scaffolder_names), 4)
    self.assertIn('clearly not awesome', scaffolder_names)
    manager.ScaffolderManager.DeregisterScaffolder(NotAwesomeTestScaffolder)

  def testRegisterScaffolders(self):
    """Test registering multiple scaffolders."""
    scaffolders = [
        AwesomeTestScaffolder, AverageTestScaffolder,
        RegistrationTestScaffolder]
    for scaffolder in scaffolders:
      manager.ScaffolderManager.DeregisterScaffolder(scaffolder)

    number_of_scaffolder_names = len(
        list(manager.ScaffolderManager.GetScaffolderNames()))
    self.assertEqual(number_of_scaffolder_names, 0)

    manager.ScaffolderManager.RegisterScaffolders(scaffolders)
    number_of_scaffolder_names = len(
        list(manager.ScaffolderManager.GetScaffolderNames()))
    self.assertEqual(number_of_scaffolder_names, 3)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The scaffolder interface classes."""
import os
import sqlite3

from typing import Dict
from typing import Iterator
from typing import Tuple

from l2tscaffolder.lib import errors
from l2tscaffolder.scaffolders import interface
from l2tscaffolder.scaffolders import plaso
from l2tscaffolder.scaffolders import manager


class SQLQuestion(interface.DictQuestion):
  """SQL Query question."""

  def ValidateAnswer(self, answer: dict):
    """Validates the answer to the SQL query question.

    The answer should be a dict that has query names as key values
    and valid SQLite commands as values. This function attempts
    to verify that the SQL commands do not have syntax errors in them
    by attempting to run it against an empty SQLite database stored
    in memory.

    Args:
      answer (dict): the answer to the question asked.

    Raises:
      errors.UnableToConfigure: if the answer is invalid.
    """
    super(SQLQuestion, self).ValidateAnswer(answer)

    temp_db = sqlite3.connect(':memory:')
    for query_name, query in answer.items():
      try:
        temp_db.execute(query)
      except ValueError as exception:
        raise errors.UnableToConfigure(
            'Unable to run query [{0:s}] with error: {1!s}'.format(
                query_name, exception))
      except (sqlite3.DatabaseError, sqlite3.OperationalError) as exception:
        if str(exception).endswith('syntax error'):
          raise errors.UnableToConfigure(
              'Unable to run query [{0:s}] with error: {1!s}'.format(
                  query_name, exception))


class PlasoSQLiteScaffolder(plaso.PlasoPluginScaffolder):
  """The plaso SQLite plugin scaffolder.

  Attributes:
    database_name (str): name of the test SQLite database for the plugin.
    database_schema (dict): a dict containing all table names (keys) and the SQL
        statement used to create the table (value), derived from the test
        database.
    data_types (dict): a dict containing all the data types generated for the
        parser, the key is the name for each SQL statement run against the
        database and the value is the data type used for each generated event
        resulting from that SQL statement.
    queries (dict): a dict containing query name and SQL statements or queries
        run against the database.
    query_columns (dict): for each SQL statement run against the database, with
        the key being query name and value being a list of all SQL column names
        that are returned for each query.
    required_tables (list): a list of all required tables needed for the plugin
        to parse this particular database.
    timestamp_columns (dict): a dict containing a list of all columns with
        timestamp values, with query names as the key.
  """

  # The name of the plugin or parser this scaffolder provides.
  NAME = 'sqlite'
  DESCRIPTION = 'Provides a scaffolder to generate a plaso SQLite plugin.'

  SCHEMA_QUERY = (
      'SELECT tbl_name, sql '
      'FROM sqlite_master '
      'WHERE type = "table" AND tbl_name != "xp_proc" '
      'AND tbl_name != "sqlite_sequence"')

  # Filenames of templates.
  TEMPLATE_PARSER_FILE = 'sqlite_plugin.jinja2'
  TEMPLATE_PARSER_TEST = 'sqlite_plugin_test.jinja2'
  TEMPLATE_FORMATTER_FILE = 'sqlite_plugin_formatter.jinja2'
  TEMPLATE_FORMATTER_TEST = 'sqlite_plugin_formatter_test.jinja2'

  # Questions, a list that contains all the needed questions that the
  # user should be prompted about before the plugin or parser is created.
  # Each element in the list needs to be an instance of BaseQuestion.
  QUESTIONS = [
      SQLQuestion(
          attribute='queries', prompt=(
            'Define the name of the SQL query (key) as well as the actual SQL '
            'queries (value) this plugin will execute'),
          key_prompt='Query Name', value_prompt='SQL Statement'),
      interface.ListQuestion(
          attribute='required_tables', prompt='List of required tables')]

  def __init__(self):
    """Initializes the plaso SQLite plugin scaffolder."""
    super(PlasoSQLiteScaffolder, self).__init__()
    self.database_name = ''
    self.database_schema = {}
    self.data_types = {}
    self.queries = {}
    self.query_columns = {}
    self.required_tables = []
    self.timestamp_columns = {}

  def _GetQueryColumns(self, query: str) -> Iterator[str]:
    """Generates column names from a SQL statement.

    Args:
      query (str): a SQL query.

    Yields:
      str: a column name extracted from a SQL statement.
    """
    _, _, query_string = query.lower().partition('select')
    query_column_string, _, _ = query_string.partition('from')

    for column in query_column_string.split(','):
      _, _, column_alias = column.partition(' as ')
      column = column_alias or column

      _, _, column = column.rpartition('.')

      if not column:
        continue

      yield column.strip()

  def _GetSchema(self, database_path: str) -> Dict[str, str]:
    """Returns the schema of a SQLite database as a dict.

    Args:
      database_path (str): full path to the SQLite database.

    Returns:
      dict: containing:
        str: name of a table in the
        str:  SQL command that was used to create the table.

    Raises:
      sqlite3.DatabaseError: if the database cannot be read.
    """
    database = sqlite3.connect(database_path)
    try:
      database.row_factory = sqlite3.Row
      cursor = database.cursor()

      sql_results = cursor.execute(self.SCHEMA_QUERY)

      schema = {
          table_name: ' '.join(query.split())
          for table_name, query in sql_results}

    except sqlite3.DatabaseError:
      database.close()
      raise

    return schema

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(PlasoSQLiteScaffolder, self).GetJinjaContext()
    context['database_name'] = self.database_name
    context['database_schema'] = self.database_schema
    context['data_types'] = self.data_types
    context['queries'] = self.queries
    context['query_columns'] = self.query_columns
    context['required_tables'] = self.required_tables
    context['timestamp_columns'] = self.timestamp_columns
    return context

  def GenerateFiles(self) -> Iterator[Tuple[str, str]]:
    """Generates files required for the SQLite plugin.

    Yields:
      tuple: file name and content of the file to be written to disk.
    """
    _, _, database_name = self.test_file.rpartition(os.sep)
    self.database_name = database_name

    self.data_types = {}
    sql_columns = {}
    timestamp_columns = {}

    for query_name, query in self.queries.items():
      self.data_types[query_name] = '{0:s}:{1:s}'.format(
          self._output_name.lower().replace('_', ':'), query_name.lower())
      timestamp_columns[query_name] = []
      sql_columns[query_name] = []

      for column in self._GetQueryColumns(query):
        if 'time' in column:
          timestamp_columns[query_name].append(column.strip())
        sql_columns[query_name].append(column)

    self.query_columns = sql_columns
    self.timestamp_columns = timestamp_columns

    self.database_schema = self._GetSchema(self.test_file)

    for name, content in super(PlasoSQLiteScaffolder, self).GenerateFiles():
      yield name, content


manager.ScaffolderManager.RegisterScaffolder(PlasoSQLiteScaffolder)



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Test class for the timesketch project."""
import os
import unittest

from l2tscaffolder.lib import definitions
from l2tscaffolder.definitions import timesketch


class TimesketchProjectTest(unittest.TestCase):
  """Test case for the timesketch project definition. """

  def testTimesketchProject(self):
    """Test timesketch project definitions."""
    test_path = os.path.join(os.path.dirname(
        os.path.dirname(os.path.dirname(__file__))), 'test_data')

    project_test = timesketch.TimesketchProject()
    self.assertEqual(project_test.NAME, definitions.DEFINITION_TIMESKETCH)

    self.assertTrue(project_test.ValidatePath(os.path.join(
        test_path, 'TimesketchPath')))

    self.assertFalse(project_test.ValidatePath(os.path.join(
        test_path, 'PlasoPath')))

    self.assertFalse(project_test.ValidatePath(os.path.join(
        test_path, 'PlasoFailPath1')))

    self.assertFalse(project_test.ValidatePath(os.path.join(
        test_path, 'PlasoFailPath2')))


if __name__ == '__main__':
  unittest.main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Test class for the definition manager."""
import unittest

from l2tscaffolder.definitions import interface
from l2tscaffolder.definitions import manager


class GoldTestProject(interface.ScaffolderDefinition):
  """Test project."""

  NAME = 'gold'

  def ValidatePath(self, root_path: str) -> bool:
    """Validates the path to the root directory of the project.

    Args:
      root_path (str): path to a project source tree.

    Returns:
      bool: true if the path contains 'gold'.
    """
    if 'gold' in root_path:
      return True

    return False


class SilverTestProject(interface.ScaffolderDefinition):
  """Test project."""

  NAME = 'silver'

  def ValidatePath(self, root_path: str) -> bool:
    """Validates the path to the root directory of the project.

    Args:
      root_path (str): path to a project source tree.

    Returns:
      bool: true if the path contains 'gold'.
    """
    if 'silver' in root_path:
      return True

    return False


class FailingTestProject(interface.ScaffolderDefinition):
  """Test project that always fails."""

  NAME = 'failing'

  def ValidatePath(self, root_path: str) -> bool:
    """Test validation.

    Args:
      root_path (str): path to a project source tree.

    Returns:
      bool: always False, for testing.
    """
    return False


class DefinitionManagerTest(unittest.TestCase):
  """Test case for the definition manager. """

  @classmethod
  def setUpClass(cls):
    """Sets up the test class."""
    # Need to make sure there are no registered projects, and can't iterate
    # directly over projects since changing the dict in the middle of an iter
    # causes errors.
    registered_definitions = list(
        manager.DefinitionManager.GetDefinitionObjects())
    for definition_object in registered_definitions:
      registered_class = type(definition_object)
      manager.DefinitionManager.DeregisterDefinition(registered_class)

    manager.DefinitionManager.RegisterDefinition(GoldTestProject)
    manager.DefinitionManager.RegisterDefinition(FailingTestProject)

  def testGetDefinitionByName(self):
    """Test getting definitions by name."""
    test_definition = manager.DefinitionManager.GetDefinitionByName(
        GoldTestProject.NAME)

    self.assertEqual(test_definition.NAME, GoldTestProject.NAME)

    test_definition = manager.DefinitionManager.GetDefinitionByName(
        'DoesNotExist')

    self.assertEqual(test_definition, None)

  def testRegisteringAndDeregistering(self):
    """Test registering and deregistering definitions."""
    definitions = list(manager.DefinitionManager.GetDefinitionNames())
    self.assertEqual(len(definitions), 2)

    manager.DefinitionManager.RegisterDefinition(SilverTestProject)
    self.assertEqual(
        len(list(manager.DefinitionManager.GetDefinitionNames())), 3)

    with self.assertRaises(KeyError):
      manager.DefinitionManager.RegisterDefinition(SilverTestProject)

    manager.DefinitionManager.DeregisterDefinition(SilverTestProject)
    self.assertEqual(
        len(list(manager.DefinitionManager.GetDefinitionNames())), 2)

  def testGetDefinitionNames(self):
    """Test getting definition names."""
    definitions = list(manager.DefinitionManager.GetDefinitionNames())

    self.assertEqual(len(definitions), 2)
    correct_definitions = ['gold', 'failing']

    self.assertSetEqual(set(definitions), set(correct_definitions))


if __name__ == '__main__':
  unittest.main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""testing the main"""
import unittest

import l2tscaffolder


class VersionTest(unittest.TestCase):
  """the version in init test"""

  def testGetVersion(self):
    """testing the get version"""
    actual = l2tscaffolder.__version__
    self.assertEqual('20181127', actual)


if __name__ == '__main__':
  unittest.main()



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests class for the scaffolder engine"""
import os
import unittest

from l2tscaffolder.definitions import interface as definition_interface
from l2tscaffolder.definitions import manager as definition_manager
from l2tscaffolder.lib import engine
from l2tscaffolder.lib import errors
from l2tscaffolder.scaffolders import interface as scaffolder_interface


class AwesomeScaffolder(scaffolder_interface.Scaffolder):
  """Awesome test scaffolder."""
  NAME = 'Awesome'
  DESCRIPTION = 'This is a really awesome thing.'
  QUESTIONS = [
      scaffolder_interface.StringQuestion('test1', 'a'),
      scaffolder_interface.StringQuestion('test2', 'a'),
      scaffolder_interface.StringQuestion('test3', 'a')]

  def __init__(self):
    super(AwesomeScaffolder, self).__init__()
    self.test1 = ''
    self.test2 = ''
    self.test3 = ''

  def GetInitFileChanges(self):
    """Generate a list of init files that need changing and the changes to them.

    Yields:
      tuple(str, str): path to the init file and the entry to add to it.
    """
    return iter(())

  def GetJinjaContext(self):
    """Returns a dict that can be used as a context for Jinja2 templates."""
    context = super(AwesomeScaffolder, self).GetJinjaContext()
    context['test1'] = self.test1
    context['test2'] = self.test2
    context['test3'] = self.test3
    return context

  def GenerateFiles(self):
    """Empty file generator."""
    return iter(())

  def GetFilesToCopy(self):
    """Empty files to copy generator."""
    return iter(())


class NotWrongDefinition(definition_interface.ScaffolderDefinition):
  """Definition for the not so wrong project."""
  NAME = 'N/A'

  def ValidatePath(self, root_path: str) -> bool:
    """Validate a path to the test project."""
    if 'wrong' in root_path:
      return False

    if 'error' in root_path:
      return False

    return True


class ScaffolderEngineTest(unittest.TestCase):
  """Test case for the scaffolder engine functions. """

  @classmethod
  def setUpClass(cls):
    definition_manager.DefinitionManager.RegisterDefinition(NotWrongDefinition)

  def testSetModuleName(self):
    """Tests setting the module name."""
    test_engine = engine.ScaffolderEngine()
    test_name = 'foobar'

    test_engine.SetModuleName(test_name)

    module_name = test_engine.module_name
    self.assertEqual(module_name, test_name.title())

    test_name = 'some module this is'
    expected_module_name = 'SomeModuleThisIs'
    expected_file_name = 'some_module_this_is'

    test_engine.SetModuleName(test_name)
    module_name = test_engine.module_name
    # pylint: disable=protected-access
    file_name = test_engine._file_name_prefix

    self.assertEqual(expected_module_name, module_name)
    self.assertEqual(expected_file_name, file_name)

  def testSetProjectRootPath(self):
    """Tests setting the root path to a project."""
    test_engine = engine.ScaffolderEngine()

    # Test a path that will fail.
    path = os.path.join(os.path.curdir, 'wrong path')
    with self.assertRaises(errors.NoValidDefinition):
      test_engine.SetProjectRootPath(path)

    path = 'this is absolutely the correct path'
    test_engine.SetProjectRootPath(path)

    # pylint: disable=protected-access
    root_path = test_engine._definition_root_path
    project = test_engine._definition

    self.assertEqual(root_path, path)
    self.assertEqual(project, NotWrongDefinition.NAME)

  def testSetScaffolder(self):
    """Tests setting the scaffolder of a scaffolder engine."""
    test_engine = engine.ScaffolderEngine()
    test_scaffolder = AwesomeScaffolder()

    test_engine.SetScaffolder(test_scaffolder)

    self.assertTrue(hasattr(test_engine, '_scaffolder'))
    # pylint: disable=protected-access
    self.assertIsInstance(
        test_engine._scaffolder, scaffolder_interface.Scaffolder)

  def testStoreScaffolderAttribute(self):
    """Tests storing attributes in a scaffolder."""
    test_engine = engine.ScaffolderEngine()
    test_scaffolder = AwesomeScaffolder()
    test_engine.SetScaffolder(test_scaffolder)

    test_string1 = 'Test String'
    test_string2 = 'Part of the scaffolder'
    test_string3 = 'I\'m stored in the scaffolder!'

    test_engine.StoreScaffolderAttribute('test1', test_string1, str)
    with self.assertRaises(errors.ScaffolderNotConfigured):
      test_scaffolder.RaiseIfNotReady()

    test_engine.StoreScaffolderAttribute('test2', test_string2, str)
    with self.assertRaises(errors.ScaffolderNotConfigured):
      test_scaffolder.RaiseIfNotReady()

    test_engine.StoreScaffolderAttribute('test3', test_string3, str)
    self.assertIsNone(test_scaffolder.RaiseIfNotReady())

    scaffolder_attributes = test_scaffolder.GetJinjaContext()
    test1_attr = scaffolder_attributes.get('test1', '')
    test2_attr = scaffolder_attributes.get('test2', '')
    test3_attr = scaffolder_attributes.get('test3', '')

    self.assertEqual(test1_attr, test_string1)
    self.assertEqual(test2_attr, test_string2)
    self.assertEqual(test3_attr, test_string3)

  def testEngineNotReady(self):
    """Tests if the engine is fully configured."""
    test_engine = engine.ScaffolderEngine()
    with self.assertRaises(errors.EngineNotConfigured):
      _ = list(test_engine.GenerateFiles())

    path = 'this is absolutely the correct path'
    test_engine.SetProjectRootPath(path)

    with self.assertRaises(errors.EngineNotConfigured):
      _ = list(test_engine.GenerateFiles())

    test_engine.SetModuleName('TestBarWithFoo')
    with self.assertRaises(errors.EngineNotConfigured):
      _ = list(test_engine.GenerateFiles())

    with self.assertRaises(errors.ScaffolderNotConfigured):
      test_engine.StoreScaffolderAttribute('test1', 'foobar', str)

    test_engine.SetScaffolder(AwesomeScaffolder())
    with self.assertRaises(errors.EngineNotConfigured):
      _ = list(test_engine.GenerateFiles())

    test_string1 = 'Test String'
    test_string2 = 'Part of the scaffolder'
    test_string3 = 'I\'m stored in the scaffolder!'
    test_engine.StoreScaffolderAttribute('test1', test_string1, str)
    test_engine.StoreScaffolderAttribute('test2', test_string2, str)
    test_engine.StoreScaffolderAttribute('test3', test_string3, str)

    self.assertListEqual(list(test_engine.GenerateFiles()), [])


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The scaffolder interface classes."""
import abc

from typing import Dict
from typing import Iterator
from typing import List
from typing import Tuple

from l2tscaffolder.lib import definitions
from l2tscaffolder.lib import errors


class BaseQuestion:
  """Scaffolder question.

  Attributes:
    attribute (str): the name of the attribute the question prompts for.
    prompt (str): help string that is displayed before the question is asked.
  """

  # The type defines the variable type this question expects
  # as an answer.
  TYPE = None

  def __init__(self, attribute: str, prompt: str):
    """Initializes the question.

    Args:
      attribute (str): the name of the attribute the question prompts for.
      prompt (str): help string that is displayed before the question is asked.
    """
    self.attribute = attribute
    self.prompt = prompt

  def _ValidateType(self, answer: object):
    """Validates the answer against the expected type.

    Args:
      answer (object): the answer to the question asked.

    Raises:
      errors.UnableToConfigure: if the answer is not the correct type.
    """
    if not isinstance(answer, self.TYPE):
      raise errors.UnableToConfigure((
          'Answer is not of the correct type. It is {0!s} '
          'instead of being {1!s}').format(type(answer)), type(self.TYPE))

  def ValidateAnswer(self, answer: object):
    """Validate an answer to a question.

    Args:
      answer (object): the answer to the question asked.

    Raises:
      errors.UnableToConfigure: if the answer is invalid.
    """
    self._ValidateType(answer)


class DictQuestion(BaseQuestion):
  """Scaffolder dict question.

  Attributes:
    attribute (str): the name of the attribute the question prompts for.
    prompt (str): help string that is displayed before the question is asked.
    key_prompt (str): the help string that is displayed before asking for each
        key.
    value_prompt (str): the help string that is displayed before asking for each
        value in the dict.
  """
  TYPE = dict

  def __init__(self, attribute, prompt, key_prompt, value_prompt):
    """Initialize the question."""
    super(DictQuestion, self).__init__(attribute, prompt)
    self.key_prompt = key_prompt
    self.value_prompt = value_prompt


class IntQuestion(BaseQuestion):
  """Scaffolder integer question."""
  TYPE = int


class ListQuestion(BaseQuestion):
  """Scaffolder list question."""
  TYPE = list


class StringQuestion(BaseQuestion):
  """Scaffolder string question."""
  TYPE = str


class Scaffolder:
  """The scaffolder interface."""

  # The name of the component/parser/submodule this scaffolder generates.
  NAME = 'base_parser'

  # One liner describing what the scaffolder generates.
  DESCRIPTION = ''

  # Define which project this particular scaffolder belongs to.
  PROJECT = definitions.DEFINITION_PLASO

  # Questions, a list that contains all the needed questions that the
  # user should be prompted about before the plugin or parser is created.
  # Each element in the list should be an instance of BaseQuestion.
  QUESTIONS = []

  def __init__(self):
    """Initializes the scaffolder."""
    super(Scaffolder, self).__init__()
    self._output_name = ''

  @abc.abstractmethod
  def GetInitFileChanges(self) -> Iterator[Tuple[str, str]]:
    """Generate a list of init files that need changing and the changes to them.

    Yields:
      tuple (str, str): path to the init file and the entry to add to it.
    """

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    return {}

  def GetQuestions(self) -> List[BaseQuestion]:
    """Returns scaffolder questions.

    Returns:
      list[BaseQuestion]: questions to prompt the user with.
    """
    return self.QUESTIONS

  @abc.abstractmethod
  def GenerateFiles(self) -> Iterator[Tuple[str, str]]:
    """Generates files this scaffolder provides.

    Yields:
      list: file name and content of the file to be written to disk.
    """

  @abc.abstractmethod
  def GetFilesToCopy(self) -> Iterator[Tuple[str, str]]:
    """Return a list of files that need to be copied.

    Yields:
      tuple (str, str): file name of source and destination.
    """

  def RaiseIfNotReady(self):
    """Checks to see if all attributes are set to start generating files.

    By default this function only checks to see if all attributes defined
    in questions and Jinja2 context have values and are not empty.

    Raises:
      ScaffolderNotConfigured: if the scaffolder is not fully configured.
    """
    for question in self.GetQuestions():
      attribute = getattr(self, question.attribute, None)

      if not attribute:
        raise errors.ScaffolderNotConfigured(
            'Attribute [{0:s}] is not set.'.format(
                question.attribute))

    for jinja_context_attribute in self.GetJinjaContext():
      if not jinja_context_attribute:
        raise errors.ScaffolderNotConfigured((
            'The required attribute for jinja2 template: [{0:s}] is not '
            'set.').format(jinja_context_attribute))

  def SetOutputName(self, output_name: str):
    """Sets the name of the output module.

    This is the name of the generated output module this scaffolder
    implements.

    Args:
      output_name (str): the name of the output that the scaffolder generates,
          whether that is an output module, plugin, parser, analyzer or
          something else.
    """
    self._output_name = output_name

  def SetAttribute(self, name: str, value: object, value_type: type):
    """Stores an attribute read from the CLI.

    Args:
      name (str): the attribute name.
      value (object): the attribute value.
      value_type (type): the attribute type.

    Raises:
      ValueError: if the value is not of the correct type.
      KeyError: If the attribute is not configured for this scaffolder.
    """
    if not hasattr(self, name):
      raise KeyError(
          'Attribute {0:s} is not configured for this scaffolder.'.format(name))

    if not isinstance(value, value_type):
      raise ValueError('Value is of type {0:s}, not {1:s}'.format(
          type(value), value_type))

    setattr(self, name, value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for command line helper."""

from __future__ import unicode_literals

import unittest

from l2tscaffolder.helpers import cli


class CLIHelperTest(unittest.TestCase):
  """Tests the command line helper"""

  def testRunCommand(self):
    """Tests that the helper can be initialized."""
    mock_responses = {'echo hi': [0, 'hi\n', '']}
    test_helper = cli.CLIHelper(mock_responses=mock_responses)
    with self.assertRaises(AttributeError):
      test_helper.RunCommand('echo hello')
    exit_code, stdout, stderr = test_helper.RunCommand('echo hi')
    self.assertEqual(exit_code, 0)
    self.assertEqual(stdout, 'hi\n')
    self.assertEqual(stderr, '')

    real_helper = cli.CLIHelper()
    exit_code, stdout, stderr = real_helper.RunCommand('echo hello')
    self.assertEqual(exit_code, 0)
    self.assertEqual(stdout, 'hello\n')
    self.assertEqual(stderr, '')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""helper for the test"""
import os

__file__ = os.path.abspath(__file__)


def TemplatePath() -> str:
  """ generating the template path for the tests

  Returns:
    str: the template path

  """
  return os.path.join(
      os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
      'l2tscaffolder', 'bll', 'templates')


def TestTemplatePath() -> str:
  """ generating the template path for the tests

  Returns:
    str: the template path

  """
  return os.path.join(
      os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
      'tests', 'test_template')


def TestDatabasePath() -> str:
  """ generating the template path for the tests

  Returns:
    str: the template path

  """
  return os.path.join(
      os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
      'tests', 'test_database')


def YapfStyleFilePath() -> str:
  """ gets the path to the yapf style file.

  Returns:
    str: the yapf file path
  """
  return os.path.join(
      os.path.dirname(os.path.dirname(__file__)),
      '.style.yapf')



# -*- coding: utf-8 -*-
"""Turbinia component scaffolder."""
import datetime
import os
import logging

from typing import Dict
from typing import Iterator
from typing import Tuple

from l2tscaffolder.lib import definitions
from l2tscaffolder.lib import mapping_helper
from l2tscaffolder.scaffolders import interface


class TurbiniaBaseScaffolder(interface.Scaffolder):
  """The Turbinia base scaffolder interface.

  Attributes:
    class_name (str): class name of the Turbinia plugin to be generated.
  """

  # The name of the plugin this scaffolder plugin provides.
  NAME = 'turbinia_base'

  # One liner describing what the scaffolder provides.
  DESCRIPTION = 'This is a scaffolder for Turbinia components'

  # Define which project this particular scaffolder belongs to.
  PROJECT = definitions.DEFINITION_TURBINIA

  # Filename of templates.
  TEMPLATE_JOB_FILE = None
  TEMPLATE_TASK_FILE = None

  def __init__(self):
    """Initializes the Turbinia scaffolder."""
    super(TurbiniaBaseScaffolder, self).__init__()
    self._plugin_path = os.path.join('turbinia', 'jobs')
    self._task_path = os.path.join('turbinia', 'workers')
    self._mapping_helper = mapping_helper.MappingHelper()

    self.class_name = ''

  def _GenerateJobFile(self) -> str:
    """Generates the job plugin file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_JOB_FILE, self.GetJinjaContext())

  def _GenerateTaskFile(self) -> str:
    """Generates the task plugin file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_TASK_FILE, self.GetJinjaContext())

  def GetInitFileChanges(self) -> Iterator[Tuple[str, str]]:
    """Generate a list of init files that need changing and the changes to them.

    Yields:
      Tuple[str, str]: path to the init file and the entry to add to it.
    """
    plugin_path = self._plugin_path.replace(os.sep, '.')
    plugin_string = 'from {0:s} import {1:s}\n'.format(
        plugin_path, self._output_name)
    plugin_init_path = os.path.join(self._plugin_path, '__init__.py')
    yield plugin_init_path, plugin_string

  def GetFilesToCopy(self) -> Iterator[Tuple[str, str]]:
    """Return a list of files that need to be copied.

    Returns:
      an empty iterator.
    """
    return iter(())

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(TurbiniaBaseScaffolder, self).GetJinjaContext()
    context['class_name'] = self.class_name
    context['plugin_name'] = self._output_name
    time_now = datetime.datetime.utcnow()
    context['year'] = time_now.year

    return context

  def GenerateFiles(self) -> Iterator[Tuple[str, str]]:
    """Generates all the files required for a Turbinia component.

    Yields:
      list[tuple]: containing:
       str: file name.
       str: file content.
    """
    plugin_name = '{0:s}.py'.format(self._output_name)

    self.class_name = self._mapping_helper.GenerateClassName(
        self._output_name)

    try:
      plugin_path = os.path.join(self._plugin_path, plugin_name)
      plugin_content = self._GenerateJobFile()
      yield plugin_path, plugin_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate component, error '
          'message: {0!s}').format(exception))

    try:
      task_path = os.path.join(self._task_path, plugin_name)
      task_content = self._GenerateTaskFile()
      yield task_path, task_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate component, error '
          'message: {0!s}').format(exception))



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Test class for the CLI frontend."""
import io
import os
import re
import unittest
from unittest.mock import MagicMock
import tempfile

from l2tscaffolder.definitions import manager as definition_manager
from l2tscaffolder.definitions import plaso
from l2tscaffolder.frontend import frontend
from l2tscaffolder.frontend import output_handler
from l2tscaffolder.scaffolders import manager as scaffolder_manager
from l2tscaffolder.scaffolders import plaso_sqlite


class TestOutputHandler(output_handler.BaseOutputHandler):
  """Test implementation of the output handler."""

  def __init__(self):
    """Initializes the test output handler."""
    self._data = []
    self._output = None

  def _GetInput(self):
    """Returns a line from the data, or an empty string if no data."""
    if not self._data:
      return ''

    return self._data.pop(0)

  def Confirm(self, text: str, default=True, abort=True):
    """Ask for a confirmation, either yes or no to a question.

    Args:
      text (str): prompts the user for a confirmation, with the given test as
          the question
      default (bool): the default for the confirmation answer. If True the
          default is Y(es), if False the default is N(o)
      abort (bool): if the program should abort if the user answer to the
          confirm prompt is no. The default is an abort.

    Returns:
       bool: False if the user entered no, True if the user entered yes
    """
    answer = self._GetInput()
    if answer.lower() in ['n', 'no']:
      return False

    return True

  def FeedLine(self, line):
    """Feed the handler with a line."""
    self._data.append(line)

  def PrintError(self, text: str):
    """A echo for errors with click.

    Args:
      text (str): the text to print
    """
    self._output.write(text)

  def PrintInfo(self, text: str):
    """A echo for information with click.

    Args:
      text (str): the text to print
    """
    self._output.write(text)

  def PrintNewLine(self):
    """A new line added to output."""
    self._output.write('\n')

  def PrintOutput(self, text: str):
    """A echo for information with click.

    Args:
      text (str): the text to print.
    """
    self._output.write(text)

  def PromptError(self, text: str) -> str:
    """A prompt for errors with click.

    Args:
      text (str): the text to prompt

    Returns:
      str: the user input
    """
    self._output.write(text)
    return self._GetInput()

  def PromptInfo(self, text: str) -> str:
    """A prompt for information with click.

    Args:
      text (str): the text to  prompt

    Returns:
      str: the user input
    """
    self._output.write(text)
    return self._GetInput()

  def PromptInfoWithDefault(
      self, text: str, input_type: object, default: object) -> str:
    """A prompt for information, with a default value and a required type.

    Args:
      text (str): the text to prompt
      input_type (object): the type of the input
      default (object): the default value

    Returns:
      str: the user input
    """
    self._output.write('{0:s} [{1:s}]'.format(text, input_type))
    return self._GetInput()

  def SetOutput(self, output_writer):
    """Set the output writer."""
    self._output = output_writer


class TestFrontend(frontend.ScaffolderFrontend):
  """Test implementation of the frontend."""

  def __init__(self, output_handler_to_use):
    """Initializes the frontend."""
    super(TestFrontend, self).__init__(output_handler_to_use)
    self._mock = MagicMock()
    self._git_helper = self._mock

  def CreateGitFeatureBranch(self, project_path: str, module_name: str):
    """Mock creating feature branch."""
    branch_name = re.sub('(?<!^)(?=[A-Z])', '_', module_name).lower()
    self._output_handler.PrintOutput(
        'Created the feature branch: {0:s} inside {1:s}'.format(
            branch_name, project_path))

  def GetProjectPath(self, definition):
    """Returns the path to the project's root folder as chosen by the user.

    Args:
      definition (definition_interface.ScaffolderDefinition): the chosen
          definition. Used to validate the project path.

    Returns:
      str: the path to the project's root folder.

    Raises:
      errors.WrongCliInput: when no valid project path has been provided.
    """
    return_value = super(TestFrontend, self).GetProjectPath(definition)
    self._git_helper = self._mock
    return return_value


class ScaffolderFrontendTest(unittest.TestCase):
  """Test case for the scaffolder frontend."""

  # Temporary directory for mocking a plaso source tree.
  root_directory = None

  @classmethod
  def setUpClass(cls):
    """Set up the test class."""
    try:
      definition_manager.DefinitionManager.RegisterDefinition(
          plaso.PlasoProject)
    except KeyError:
      pass

    try:
      scaffolder_manager.ScaffolderManager.RegisterScaffolder(
          plaso_sqlite.PlasoSQLiteScaffolder)
    except KeyError:
      pass
    cls.root_directory = tempfile.TemporaryDirectory()
    root_dir_name = cls.root_directory.name
    os.mkdir(os.path.join(root_dir_name, 'plaso'))
    os.mkdir(os.path.join(root_dir_name, 'plaso', 'parsers'))
    os.mkdir(os.path.join(root_dir_name, 'plaso', 'test_data'))
    os.mkdir(os.path.join(root_dir_name, 'plaso', 'tests'))
    os.mkdir(os.path.join(root_dir_name, 'plaso', 'parsers', 'sqlite_plugins'))
    fw = open(os.path.join(root_dir_name, 'plaso.ini'), 'w')
    fw.write('\n')
    fw.close()

  @classmethod
  def tearDownClass(cls):
    """Cleans up after running tests."""
    cls.root_directory.cleanup()

  def testFrontend(self):
    """Tests the frontend.

    This test runs the entire front-end and tests to see if
    files are generated as expected.
    """
    string_buffer = io.StringIO()
    test_output_handler = TestOutputHandler()
    test_output_handler.SetOutput(string_buffer)

    test_output_handler.FeedLine(self.root_directory.name)
    test_output_handler.FeedLine('foobar test')
    test_output_handler.FeedLine('0')
    test_output_handler.FeedLine('FooQuery')
    test_output_handler.FeedLine('SELECT foo FROM bar;')
    test_output_handler.FeedLine('n')
    test_output_handler.FeedLine('bar')
    test_output_handler.FeedLine('n')
    cwd = os.getcwd()
    test_output_handler.FeedLine(
        os.path.join(cwd, 'test_data', 'test_sqlite.db'))
    test_output_handler.FeedLine('Y')

    test_frontend = TestFrontend(test_output_handler)
    test_frontend.Start('plaso')

    string_buffer.seek(0)
    lines = string_buffer.read().split('\n')
    first_line = lines[0]

    expected_string = (
        '== Starting the scaffolder ==Gathering required information.')
    self.assertEqual(first_line.strip(), expected_string)

    expected_string = 'written to disk.'
    last_line = lines[-1]
    last_part = last_line[-len(expected_string):]
    self.assertEqual(last_part, expected_string)

    self.assertTrue(os.path.isfile(os.path.join(
        self.root_directory.name, 'plaso', 'parsers', 'sqlite_plugins',
        'foobar_test.py')))
    self.assertTrue(os.path.isfile(os.path.join(
        self.root_directory.name, 'tests', 'parsers', 'sqlite_plugins',
        'foobar_test.py')))

    self.assertTrue(os.path.isfile(os.path.join(
        self.root_directory.name, 'test_data', 'test_sqlite.db')))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""The l2t scaffolder tool."""
import click

from l2tscaffolder.frontend import cli_output_handler
from l2tscaffolder.frontend import frontend


@click.command()
@click.argument(
    'definition', envvar='SCAFFOLDER_DEFINITION', type=str, default='')
def StartCLI(definition):
  """Generates templates for parser and plugins for l2t developers.

  This is a l2t scaffolder, used to generate templates for all plugin
  and parser creation for l2t tools.
  """
  output_handler = cli_output_handler.OutputHandlerClick()
  cli = frontend.ScaffolderFrontend(output_handler)

  cli.Start(definition)


if __name__ == '__main__':
  StartCLI()  # pylint: disable=no-value-for-parameter



# -*- coding: utf-8 -*-
"""The Timesketch definition class."""

import os

from l2tscaffolder.lib import definitions
from l2tscaffolder.definitions import interface
from l2tscaffolder.definitions import manager

class TimesketchProject(interface.ScaffolderDefinition):
  """Timesketch project definition."""

  NAME = definitions.DEFINITION_TIMESKETCH

  def ValidatePath(self, root_path: str) -> bool:
    """Validates the path to a Timesketch development tree.

    Args:
      root_path (str): the path to the root of the project directory.

    Returns:
      bool: whether the given path is the correct root path to a Timesketch
          development tree.
    """
    if not os.path.isdir(root_path):
      return False

    if not os.path.isdir(os.path.join(root_path, 'timesketch')):
      return False

    if not os.path.isfile(os.path.join(root_path, 'timesketch.conf')):
      return False

    if not os.path.isdir(os.path.join(root_path, 'timesketch', 'views')):
      return False

    return True


manager.DefinitionManager.RegisterDefinition(TimesketchProject)



# -*- coding: utf-8 -*-
"""Turbinia job and task scaffolder."""
from l2tscaffolder.scaffolders import turbinia
from l2tscaffolder.scaffolders import manager


class TurbiniaJobTaskScaffolder(turbinia.TurbiniaBaseScaffolder):
  """The Turbinia job and task plugin scaffolder."""

  # The name of the plugin or parser this scaffolder provides.
  NAME = 'turbinia_job_and_task'
  DESCRIPTION = (
      'Provides a scaffolder to generate a Turbinia job and task plugins.')

  # Filenames of templates.
  TEMPLATE_JOB_FILE = 'turbinia_job.jinja2'
  TEMPLATE_TASK_FILE = 'turbinia_task.jinja2'


manager.ScaffolderManager.RegisterScaffolder(TurbiniaJobTaskScaffolder)



# -*- coding: utf-8 -*-
"""The file handler."""
import os
import pathlib
import shutil

from l2tscaffolder.lib import errors


class FileHandler:
  """Handles the creation of files."""

  def AddImportToInit(self, path: str, entry: str):
    """Adds an import into an init file in the correct order.

    Args:
      path (str): path to the __init__ file.
      entry (str): the import statement.
    """
    if not os.path.isfile(path):
      return

    with open(path, 'r') as file_object:
      lines = file_object.readlines()

    line_index = 0
    for index, line in enumerate(lines):
      if not line.startswith('from '):
        continue
      if line > entry:
        line_index = index
        break

    if line_index:
      lines.insert(line_index, entry)
    else:
      lines.append(entry)

    with open(path, 'w') as file_object:
      for line in lines:
        file_object.write(line)

  @classmethod
  def CreateFilePath(cls, path: str, name: str, extension: str) -> str:
    """Creates the file path from the directory path, filename and suffix.

    Args:
      path (str): path to the file directory.
      name (str): filename.
      extension (str): file extension.

    Returns:
      str: the path to the file.
    """
    file_name = '{0:s}.{1:s}'.format(name, extension)
    return os.path.join(path, file_name)

  @classmethod
  def _CreateFolder(cls, directory_path: str):
    """Creates a folder.

     This function should only to be called if the target folder does not yet
     exist or there will be an exception.

     Args:
       directory_path (str): path to the directory to create.
     """
    os.makedirs(directory_path)

  def CreateFile(
      self, directory_path: str, file_name: str,
      filename_extension: str) -> str:
    """Creates a empty file.

    Args:
      directory_path (str): path to the directory the file should be created in.
      file_name (str): name of the new file.
      filename_extension (str): extension of the new file.

    Returns:
      str: path of the created file
    """
    file_path = self.CreateFilePath(
        directory_path, file_name, filename_extension)

    if not os.path.exists(directory_path):
      self._CreateFolder(directory_path)

    pathlib.Path(file_path).touch()
    return file_path

  def CreateFileFromPath(self, file_path: str) -> str:
    """Creates a empty file.

    Args:
      file_path (str): path to the file.

    Returns:
      str: the path of the created file
    """
    _ = self.CreateFolderForFilePathIfNotExist(file_path)
    pathlib.Path(file_path).touch()
    return file_path

  def CopyFile(self, source: str, destination: str) -> str:
    """Copies a file.

    Args:
      source (str): path of the file to copy
      destination (str): path to copy the file to.

    Returns:
      str: the path of the copied file

    Raises:
      errors.FileHandlingError: when file copy operation fails.
    """
    _ = self.CreateFolderForFilePathIfNotExist(destination)
    try:
      shutil.copyfile(source, destination)
    except shutil.SameFileError as exception:
      raise errors.FileHandlingError((
          'Unable to copy file source and dest are the same files. '
          'Original error message: {0:s}').format(exception))
    except OSError as exception:
      raise errors.FileHandlingError(
          'Unable to copy file, error message: {0:s}'.format(exception))

    return destination

  def CreateOrModifyFileWithContent(self, source: str, content: str):
    """Adds content to a file and create the file and path if non existing.

    Args:
      source (str): path of the file to edit.
      content (str): content to append to the file.

    Returns:
      str: path of the edited file.
    """
    _ = self.CreateFolderForFilePathIfNotExist(source)
    return self.AddContent(source, content)

  def AddContent(self, source: str, content: str) -> str:
    """Adds content to a file and create file if non existing.

    Args:
      source (str): path of the file to edit.
      content (str): content to append to the file.

    Returns:
      str: path of the edited file.
    """
    _ = self.CreateFolderForFilePathIfNotExist(source)
    with open(source, 'a') as file_object:
      file_object.write(str(content))

    return source

  def CreateFolderForFilePathIfNotExist(self, file_path: str) -> str:
    """Creates folders for the given file if it does not exist.

    Args:
      file_path (str):  path to the file

    Returns:
      str: directory path of the created directory
    """
    directory_path = os.path.dirname(file_path)
    if not os.path.exists(directory_path):
      self._CreateFolder(directory_path)
    return directory_path



# -*- coding: utf-8 -*-
"""This file contains the error classes."""


class Error(Exception):
  """Base error class."""


class EngineNotConfigured(Error):
  """Raised when the scaffolder engine has not been configured correctly."""


class FileHandlingError(Error):
  """Raised when the file handler is unable to do file operation."""


class NoValidDefinition(Error):
  """Raised when no valid project definition has been identified."""


class ScaffolderNotConfigured(Error):
  """Raised when the scaffolder has not been configured correctly."""


class UnableToConfigure(Error):
  """Raised when the scaffolder tool has issues with configuration."""


class WrongCliInput(Error):
  """Raised when wrong input is entered into the CLI."""



# -*- coding: utf-8 -*-
"""The output file handler.

This file defines the interface of how an output handler should operate.
An output handler is used as a UI element, for two things:
1. Relay information back to the user.
2. Gather input from an end user and presenting it back to the tool.
"""
import abc


class BaseOutputHandler:
  """Interface for the output handler."""

  @abc.abstractmethod
  def Confirm(self, text: str, default=True, abort=True):
    """Returns a bool from a yes/no question presented to the end user.

    Args:
      text (str): the question presented to the end user.
      default (bool): the default for the confirmation answer. If True the
          default is Y(es), if False the default is N(o)
      abort (bool): if the program should abort if the user answer to the
          confirm prompt is no. The default is an abort.

    Returns:
       bool: False if the user entered no, True if the user entered yes
    """

  @abc.abstractmethod
  def PrintError(self, text: str):
    """Presents an error message.

    Args:
      text (str): the error message to present.
    """

  @abc.abstractmethod
  def PrintInfo(self, text: str):
    """Presents the user with an informational text.

    Args:
      text (str): the text to present.
    """

  @abc.abstractmethod
  def PrintNewLine(self):
    """Adds a new or blank line to the output."""

  @abc.abstractmethod
  def PrintOutput(self, text: str):
    """Presents the user with output from the tool.

    Args:
      text (str): the text to present the user with.
    """

  @abc.abstractmethod
  def PromptError(self, text: str) -> str:
    """Presents the user with an error message prompt and return back the answer.

    Args:
      text (str): the text to prompt

    Returns:
      str: the user input.
    """

  @abc.abstractmethod
  def PromptInfo(self, text: str) -> str:
    """Presents the user with a message prompt and return back the answer.

    Args:
      text (str): the text to prompt

    Returns:
      str: the user input.
    """

  @abc.abstractmethod
  def PromptInfoWithDefault(
      self, text: str, input_type: type, default: object) -> str:
    """Presents the user with a prompt with a default return value and a type.

    The prompt can have a default value to be chosen as well as a defined type
    of the returned data.

    Args:
      text (str): the text to prompt
      input_type (type): the type of the input
      default (object): the default value

    Returns:
      object: the user input, using the supplied input type.
    """



# -*- coding: utf-8 -*-
"""The scaffolder engine."""
import logging
import os

from typing import Iterator
from typing import Type

from l2tscaffolder.definitions import manager
from l2tscaffolder.lib import errors
from l2tscaffolder.lib import file_handler
from l2tscaffolder.scaffolders import interface as scaffolder_interface


class ScaffolderEngine:
  """The engine, responsible for file handling and setting up scaffolders."""

  def __init__(self):
    """Initializes the engine."""
    super(ScaffolderEngine, self).__init__()
    self._attributes = {}
    self._definition = ''
    self._definition_root_path = ''
    self._file_handler = file_handler.FileHandler()
    self._file_name_prefix = ''
    self._scaffolder = None
    self.module_name = ''

  def _RaiseIfNotReady(self):
    """Checks to see if all attributes are set to start generating files.

    Raises:
      errors.EngineNotConfigured: when the engine is not fully configured.
    """
    if not self._definition_root_path:
      raise errors.EngineNotConfigured(
          'The path to the project root is not properly configured.')

    if not self.module_name:
      raise errors.EngineNotConfigured('Module name has not been configured.')

    if not self._scaffolder:
      raise errors.EngineNotConfigured('Scaffolder object not yet set.')

    try:
      self._scaffolder.RaiseIfNotReady()
    except errors.ScaffolderNotConfigured as exception:
      raise errors.EngineNotConfigured(exception)

  def GenerateFiles(self) -> Iterator[str]:
    """Generates needed files.

    Raises:
      errors.EngineNotConfigured: when not all attributes have been configured.

    Yields:
      str: the full path to a file that was generated and written to disk.
    """
    self._RaiseIfNotReady()

    self._scaffolder.SetOutputName(self._file_name_prefix)

    for file_source, file_destination in self._scaffolder.GetFilesToCopy():
      if os.path.isfile(file_source):
        full_path = os.path.join(self._definition_root_path, file_destination)
        try:
          written_file = self._file_handler.CopyFile(file_source, full_path)
          yield written_file
        except errors.FileHandlingError as exception:
          logging.error(
              'Unable to copy file: {0:s} to {1:s} with error: {2!s}'.format(
                  file_source, full_path, exception))

    for file_path, content in self._scaffolder.GenerateFiles():
      full_path = os.path.join(self._definition_root_path, file_path)
      yield self._file_handler.AddContent(full_path, content)

    for file_path, content in self._scaffolder.GetInitFileChanges():
      full_path = os.path.join(self._definition_root_path, file_path)
      self._file_handler.AddImportToInit(full_path, content)
      yield full_path

  def SetModuleName(self, module_name: str):
    """Sets the module name as chosen by the user.

    Args:
      module_name (str): name of the module to be generated by the scaffolder.
    """
    file_name_prefix = module_name.replace(' ', '_')
    file_name_prefix = file_name_prefix.lower()
    module_name = file_name_prefix.replace('_', ' ')
    module_name = module_name.title()
    module_name = module_name.replace(' ', '')
    self._file_name_prefix = file_name_prefix
    self.module_name = module_name

  def SetScaffolder(self, scaffolder: scaffolder_interface.Scaffolder):
    """Stores and initializes the scaffolder object in the engine.

    Args:
      scaffolder (scaffolder_interface.Scaffolder): the scaffolder class
          that the engine will use to generate files.
    """
    self._scaffolder = scaffolder

  def SetProjectRootPath(self, root_path: str):
    """Sets the path to the root of the project tree.

    Raises:
      errors.NoValidDefinition: when root path is not identified as a valid
          definition path.
    """
    for definition in manager.DefinitionManager.GetDefinitionObjects():
      if definition.ValidatePath(root_path):
        self._definition = definition.NAME
        self._definition_root_path = root_path
        return

    raise errors.NoValidDefinition('No valid definition has been identified.')

  def StoreScaffolderAttribute(
      self, name: str, value: object, value_type: Type):
    """Stores an attribute read from the CLI.

    Args:
      name (str): the attribute name.
      value (object): the attribute value.
      value_type (type): the attribute type.

    Raises:
      KeyError: if the attribute name is already defined.
      ScaffolderNotConfigured: if the scaffolder has not yet been set.
      ValueError: if the value is not of the correct type.
    """
    if not self._scaffolder:
      raise errors.ScaffolderNotConfigured('Scaffolder has not yet been set.')

    self._scaffolder.SetAttribute(name, value, value_type)



# -*- coding: utf-8 -*-
"""Timesketch scaffolder that generates analyzer plugins."""
import os
import logging

from typing import Dict
from typing import Iterator
from typing import Tuple

from l2tscaffolder.lib import definitions
from l2tscaffolder.lib import mapping_helper
from l2tscaffolder.scaffolders import interface


class TimesketchBaseScaffolder(interface.Scaffolder):
  """The Timesketch base scaffolder interface.

  Attributes:
    class_name (str): class name of the Timesketch analyzer to be generated.
  """

  # The name of the plugin this scaffolder plugin provides.
  NAME = 'timesketch_base'

  # One liner describing what the scaffolder provides.
  DESCRIPTION = 'This is a scaffolder for Timesketch analyzers'

  # Define which project this particular scaffolder belongs to.
  PROJECT = definitions.DEFINITION_TIMESKETCH

  # Filename of templates.
  TEMPLATE_PLUGIN_FILE = ''
  TEMPLATE_PLUGIN_TEST = ''

  # Questions, a list that contains all the needed questions that the
  # user should be prompted about before the plugin or parser is created.
  # Each element in the list should be of the named tuple question.
  QUESTIONS = []

  def __init__(self):
    """Initializes the Timesketch scaffolder."""
    super(TimesketchBaseScaffolder, self).__init__()
    self._plugin_path = os.path.join('timesketch', 'lib', 'analyzers')
    self._plugin_test_path = os.path.join('timesketch', 'lib', 'analyzers')
    # Timesketch uses 4 spaces instead of 2, thus we need to set a different
    # formatter.
    self._mapping_helper = mapping_helper.MappingHelper(
        formatter_path='.style.ts.yapf')

    self.class_name = ''

  def _GeneratePlugin(self) -> str:
    """Generates the plugin file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_PLUGIN_FILE, self.GetJinjaContext())

  def _GeneratePluginTest(self) -> str:
    """Generates the plugin test file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_PLUGIN_TEST, self.GetJinjaContext())

  def GetInitFileChanges(self) -> Iterator[Tuple[str, str]]:
    """Generate a list of init files that need changing and the changes to them.

    Yields:
      Tuple[str, str]: path to the init file and the entry to add to it.
    """
    plugin_path = self._plugin_path.replace(os.sep, '.')
    plugin_string = 'from {0:s} import {1:s}\n'.format(
        plugin_path, self._output_name)
    plugin_init_path = os.path.join(self._plugin_path, '__init__.py')
    yield plugin_init_path, plugin_string

  def GetFilesToCopy(self) -> Iterator[Tuple[str, str]]:
    """Return a list of files that need to be copied.

    Returns:
      an empty iterator.
    """
    return iter(())

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(TimesketchBaseScaffolder, self).GetJinjaContext()
    context['class_name'] = self.class_name
    context['plugin_name'] = self._output_name

    return context

  def GenerateFiles(self) -> Iterator[Tuple[str, str]]:
    """Generates all the files required for a Timesketch analyzer plugin.

    Yields:
      list[tuple]: containing:
       str: file name.
       str: file content.
    """
    plugin_name = '{0:s}.py'.format(self._output_name)

    self.class_name = self._mapping_helper.GenerateClassName(
        self._output_name)

    try:
      plugin_path = os.path.join(self._plugin_path, plugin_name)
      plugin_content = self._GeneratePlugin()
      yield plugin_path, plugin_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate plugin, error '
          'message: {0!s}').format(exception))

    test_file_name = '{0:s}_test.py'.format(self._output_name)
    test_path = os.path.join(self._plugin_test_path, test_file_name)
    try:
      test_content = self._GeneratePluginTest()
      yield test_path, test_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate plugin test, error '
          'message: {0!s}').format(exception))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the git helper."""

import unittest

from l2tscaffolder.helpers import git


class GitHelperTest(unittest.TestCase):
  """Tests the git helper"""

  def testInitialize(self):
    """Tests that the helper can be initialized."""
    helper = git.GitHelper(
        'https://github.com/log2timeline/l2tdevtools.git')
    self.assertIsNotNone(helper)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Timesketch index analyzer scaffolder."""
from l2tscaffolder.scaffolders import timesketch
from l2tscaffolder.scaffolders import manager


class TimesketchIndexScaffolder(timesketch.TimesketchBaseScaffolder):
  """The Timesketch index analyzer plugin scaffolder."""

  # The name of the plugin or parser this scaffolder provides.
  NAME = 'index_analyzer'
  DESCRIPTION = (
      'Provides a scaffolder to generate a Timesketch index analyzer plugin.')

  # Filenames of templates.
  TEMPLATE_PLUGIN_FILE = 'ts_index_analyzer.jinja2'
  TEMPLATE_PLUGIN_TEST = 'ts_index_analyzer_test.jinja2'


manager.ScaffolderManager.RegisterScaffolder(TimesketchIndexScaffolder)



# -*- coding: utf-8 -*-
"""Plaso scaffolder that generates plaso parser and plugins."""
import os
import logging

from typing import Dict
from typing import Iterator
from typing import List
from typing import Tuple

from l2tscaffolder.lib import definitions
from l2tscaffolder.lib import errors
from l2tscaffolder.lib import mapping_helper
from l2tscaffolder.scaffolders import interface


class TestFileQuestion(interface.StringQuestion):
  """Test file question."""

  def ValidateAnswer(self, answer: str):
    """Validates the answer to the test file question.

    Args:
      answer (str): path to a test file.

    Raises:
      errors.UnableToConfigure: if the answer is invalid.
    """
    super(TestFileQuestion, self).ValidateAnswer(answer)

    if not os.path.isfile(answer):
      raise errors.UnableToConfigure(
          'The test file {0:s} does not exist.'.format(answer))


class PlasoBaseScaffolder(interface.Scaffolder):
  """The plaso base scaffolder interface.

  Attributes:
    class_name (str): class name of the plaso parser or plugin to be generated.
    test_file (str): name of the file used for testing the parser or plugin.
    test_file_path (str): path to the test file.
  """

  # The name of the plugin or parser this scaffolder plugin provides.
  NAME = 'plaso_base'

  # One liner describing what the scaffolder provides.
  DESCRIPTION = 'This is a scaffolder for plaso parsers and/or plugins'

  # Define which project this particular scaffolder belongs to.
  PROJECT = definitions.DEFINITION_PLASO

  # Filename of templates.
  TEMPLATE_PARSER_FILE = 'generic_plaso_parser.jinja2'
  TEMPLATE_PARSER_TEST = 'generic_plaso_parser_test.jinja2'
  TEMPLATE_FORMATTER_FILE = 'generic__plaso_formatter.jinja2'
  TEMPLATE_FORMATTER_TEST = 'generic_plaso_formatter_test.jinja2'

  # Questions, a list that contains all the needed questions that the
  # user should be prompted about before the plugin or parser is created.
  # Each element in the list should be of the named tuple question.
  QUESTIONS = []

  def __init__(self):
    """Initializes the plaso scaffolder."""
    super(PlasoBaseScaffolder, self).__init__()
    self._formatter_path = os.path.join('plaso', 'formatters')
    self._formatter_test_path = os.path.join('tests', 'formatters')
    self._parser_path = os.path.join('plaso', 'parsers')
    self._parser_test_path = os.path.join('tests', 'parsers')
    self._mapping_helper = mapping_helper.MappingHelper()

    self.class_name = ''
    self.test_file = ''
    self.test_file_path = ''

  def _GenerateFormatter(self) -> str:
    """Generates the formatter file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_FORMATTER_FILE, self.GetJinjaContext())

  def _GenerateFormatterTest(self) -> str:
    """Generates the formatter test file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_FORMATTER_TEST, self.GetJinjaContext())

  def _GenerateParser(self) -> str:
    """Generates the parser file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_PARSER_FILE, self.GetJinjaContext())

  def _GenerateParserTest(self) -> str:
    """Generates the parser test file."""
    return self._mapping_helper.RenderTemplate(
        self.TEMPLATE_PARSER_TEST, self.GetJinjaContext())

  def GetInitFileChanges(self) -> Iterator[Tuple[str, str]]:
    """Generate a list of init files that need changing and the changes to them.

    Yields:
      Tuple[str, str]: path to the init file and the entry to add to it.
    """
    formatter_string = 'from plaso.formatters import {0:s}\n'.format(
        self._output_name)
    formatter_init_path = os.path.join(self._formatter_path, '__init__.py')
    yield formatter_init_path, formatter_string

    parser_string = 'from {0:s} import {1:s}\n'.format(
        self._parser_path.replace(os.sep, '.'), self._output_name)
    parser_init_path = os.path.join(self._parser_path, '__init__.py')
    yield parser_init_path, parser_string

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(PlasoBaseScaffolder, self).GetJinjaContext()

    context['class_name'] = self.class_name
    context['test_file'] = self.test_file
    context['test_file_path'] = self.test_file_path

    return context

  def GetQuestions(self) -> List[interface.BaseQuestion]:
    """Returns scaffolder questions as well as adding plaso related ones.

    Returns:
      list[interface.BaseQuestion]: questions to prompt the user with.
    """
    test_file_question = TestFileQuestion(
        'test_file',
        'Absolute or relative path to the file that will be used for tests.')
    questions = self.QUESTIONS
    questions.append(test_file_question)
    return questions

  def GenerateFiles(self) -> Iterator[Tuple[str, str]]:
    """Generates all the files required for a plaso parser or a plugin.

    Yields:
      list[tuple]: containing:
       str: file name.
       str: file content.
    """
    parser_name = '{0:s}.py'.format(self._output_name)

    self.class_name = self._mapping_helper.GenerateClassName(
        self._output_name)

    try:
      parser_path = os.path.join(self._parser_path, parser_name)
      parser_content = self._GenerateParser()
      yield parser_path, parser_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate parser, error '
          'message: {0!s}').format(exception))

    test_path = os.path.join(self._parser_test_path, parser_name)
    try:
      test_content = self._GenerateParserTest()
      yield test_path, test_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate parser test, error '
          'message: {0!s}').format(exception))

    formatter_path = os.path.join(self._formatter_path, parser_name)
    try:
      formatter_content = self._GenerateFormatter()
      yield formatter_path, formatter_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate formatter, error '
          'message: {0!s}').format(exception))

    formatter_test_path = os.path.join(self._formatter_test_path, parser_name)
    try:
      formatter_test_content = self._GenerateFormatterTest()
      yield formatter_test_path, formatter_test_content
    except SyntaxError as exception:
      logging.error((
          'Syntax error while attempting to generate formatter test, error '
          'message: {0!s}').format(exception))

  # pylint raises issues with OSError being raised and not documented, but it is
  # not raised.
  # pylint: disable=missing-raises-doc
  def GetFilesToCopy(self) -> Iterator[Tuple[str, str]]:
    """Return a list of files that need to be copied.

    Raises:
      IOError: when the test file does not exist.

    Yields:
      tuple: containing:
        str: file name of source.
        str: file name of destination.
    """
    if not self.test_file:
      raise IOError('A plaso parser cannot be generated without a test file.')

    if not os.path.isfile(self.test_file):
      raise IOError('Test file [{0:s}] does not exist.'.format(self.test_file))

    test_file_name = os.path.basename(self.test_file)
    self.test_file_path = os.path.join('test_data', test_file_name)

    yield self.test_file, self.test_file_path

  def RaiseIfNotReady(self):
    """Checks to see if all attributes are set to start generating files.

    Raises:
      ScaffolderNotConfigured: if the scaffolder is not fully configured.
    """
    super(PlasoBaseScaffolder, self).RaiseIfNotReady()
    if not os.path.isfile(self.test_file):
      errors.ScaffolderNotConfigured(
          'Test file path is incorrect, file does not exist.')


class PlasoPluginScaffolder(PlasoBaseScaffolder):
  """Scaffolder for generating plaso plugins."""

  def __init__(self):
    """Initializes the plaso plugin scaffolder."""
    super(PlasoPluginScaffolder, self).__init__()
    plugin_path_name = '{0:s}_plugins'.format(self.NAME)
    self._parser_path = os.path.join(self._parser_path, plugin_path_name)
    self._parser_test_path = os.path.join(
        self._parser_test_path, plugin_path_name)

    self.plugin_name = ''

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(PlasoPluginScaffolder, self).GetJinjaContext()
    context['plugin_name'] = self._output_name
    return context


class PlasoParserScaffolder(PlasoBaseScaffolder):
  """Scaffolder for generating plaso parsers.

  Attributes:
    parser_name(str): name of the parser to be generated.
  """

  def __init__(self):
    """Initializes the plaso plugin scaffolder."""
    super(PlasoParserScaffolder, self).__init__()
    self.parser_name = ''

  def GetJinjaContext(self) -> Dict[str, object]:
    """Returns a dict that can be used as a context for Jinja2 templates.

    Returns:
      dict: containing:
        str: name of Jinja argument.
        object: Jinja argument value.
    """
    context = super(PlasoParserScaffolder, self).GetJinjaContext()
    context['parser_name'] = self._output_name
    return context



# -*- coding: utf-8 -*-
"""The plaso definition class."""

import os

from l2tscaffolder.lib import definitions
from l2tscaffolder.definitions import interface
from l2tscaffolder.definitions import manager

class PlasoProject(interface.ScaffolderDefinition):
  """Plaso project definition."""

  NAME = definitions.DEFINITION_PLASO

  def ValidatePath(self, root_path: str) -> bool:
    """Validates the path to a Plaso development tree.

    Args:
      root_path (str): the path to the root of the project directory.

    Returns:
      bool: whether the given path is the correct root path to a Plaso
          development tree.
    """
    if not os.path.isdir(root_path):
      return False

    if not os.path.isdir(os.path.join(root_path, 'plaso')):
      return False

    if not os.path.isfile(os.path.join(root_path, 'plaso.ini')):
      return False

    if not os.path.isdir(os.path.join(root_path, 'plaso', 'parsers')):
      return False

    return True


manager.DefinitionManager.RegisterDefinition(PlasoProject)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""This is the setup file for the project."""
import os
import glob

from setuptools import find_packages
from setuptools import setup

import l2tscaffolder

with open('README.md', 'r') as fh:
  long_description = fh.read()

setup(
    name='l2tscaffolder',
    version=l2tscaffolder.__version__,
    description=(
        'Scaffolder project for l2t, helping to bootstrap l2t development.'),
    license='Apache License, Version 2.0',
    url='https://github.com/log2timeline/l2tscaffolder',
    long_description=long_description,
    long_description_content_type='text/markdown',
    maintainer='Log2Timeline maintainers',
    maintainer_email='log2timeline-maintainers@googlegroups.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'l2tscaffolder.templates': ['*.jinja2'],
        'l2tscaffolder':['.*.yapf']},
    install_requires=['Click>=6.7',
                      'setuptools>=35.0.2',
                      'jinja2>=2.9.6',
                      'colorama>=0.3.7',
                      'yapf==0.22',
                      'pexpect>=4.2.1'],
    scripts=glob.glob(os.path.join('tools', '[a-z]*.py')),
    python_requires='>=3',
    keywords="plaso l2t scaffolder log2timeline turbinia timesketch",
)



# -*- coding: utf-8 -*-
"""Git helper for the scaffolder project.

This file provides a class to assist with git operations.
"""
import os
import re

from typing import Tuple

from l2tscaffolder.helpers import cli
from l2tscaffolder.lib import errors

class GitHelper(cli.CLIHelper):
  """Helper class for git operations.

  Attributes:
    project_path: path to the git project folder.
  """

  def __init__(self, project_path: str):
    """Initializes the git helper.

    Arguments:
      project_path (str): the path to the git project folder.
    """
    super(GitHelper, self).__init__()
    self.project_path = project_path
    self._cwd = os.getcwd()

  def AddFileToTrack(self, file_path: str):
    """Add a file to those that are tracked by the git repo.

    Args:
      file_path (str): path to the file to be added to tracked
          files by this git repo.

    Raises:
      errors.UnableToConfigure: when the tool is not able to add
          newly added files to the git repo.
    """
    command = 'git add {0:s}'.format(file_path)
    exit_code, output, error = self.RunCommand(command)
    if exit_code != 0:
      raise errors.UnableToConfigure((
          'Unable to add files to git branch, output of "git add" '
          'command is [{0:s}] with the error: {1:s}'.format(output, error)))

  def HasBranch(self, branch_name: str) -> bool:
    """Tests for the existence of a specific branch.

    Args:
      branch_name (str): the name of the branch to test for.

    Returns:
      bool: True if the branch exists.
    """
    command = 'git show-ref --verify --quiet refs/heads/"{0:s}"'.format(
        branch_name)
    exit_code, _, _ = self.RunCommand(command)
    if exit_code == 0:
      return True

    return False

  def GenerateBranchName(self, module_name: str) -> str:
    """Generates a git branch name.

    Args:
      module_name (str): module name to generate a git branch name from.

    Returns:
      str: git branch name.
    """
    branch_name = re.sub('(?<!^)(?=[A-Z])', '_', module_name)
    branch_name = branch_name.lower()
    return branch_name

  def GetActiveBranch(self) -> str:
    """Determines the active branch of the git project.

    Returns:
      str: the active branch of the git project.

    Raises:
      errors.UnableToConfigure: when the tool is not able to get
          the active branch of the git project.
    """
    command = 'git branch --list --no-color'
    exit_code, output, error = self.RunCommand(command)

    if exit_code != 0:
      raise errors.UnableToConfigure((
          'Unable to get the active git branch, with error message '
          '{0:s}').format(error))

    for line in output.split('\n'):
      if line.startswith('*'):
        _, _, line_string = line.partition('*')
        return line_string.strip()
    raise errors.UnableToConfigure('Unable to determine the active git branch')

  def RunCommand(self, command: str) -> Tuple[int, str, str]:
    """Runs a command.

    Args:
      command (str): command to run.

    Returns:
      tuple[int, str, str]: exit code, output that was written to stdout
          and stderr.
    """
    os.chdir(self.project_path)
    exit_code, output, error = super(GitHelper, self).RunCommand(command)
    os.chdir(self._cwd)
    return exit_code, output, error

  def SwitchToBranch(self, branch: str) -> int:
    """Switches the git branch and returns the exit code of the command.

    Arguments:
      branch (str): the name of the git branch.

    Returns:
      int: the exit code from the git command.
    """
    command = 'git checkout {0:s}'.format(branch)
    exit_code, _, _ = self.RunCommand(command)

    return exit_code

  def CreateBranch(self, branch: str) -> int:
    """Creates a git branch and returns the exit code of the command.

    Arguments:
      branch (str): the name of the git branch.

    Returns:
      int: the exit code from the git command.
    """
    command = 'git branch {0:s}'.format(branch)
    exit_code, _, _ = self.RunCommand(command)

    return exit_code



# -*- coding: utf-8 -*-
"""Helper methods for mapping."""
import os
import jinja2

from l2tscaffolder.lib import code_formatter


class MappingHelper:
  """Mapping helper for scaffolders."""

  _DEFAULT_PATH_FORMATTER = '.style.yapf'
  _DEFAULT_PATH_TEMPLATE = 'templates'

  def __init__(self, template_path: str='', formatter_path: str=''):
    """Initializes the mapping helper class.

    Args:
        template_path (Optional[str]):  file path to the templates diretory,
            relative to the path to the tool. If none provided will use the
            default path.
        formatter_path (Optional[str]):  file path of the formatter, relative
            to the path to the tool. If none provided will use the default
            path.
    """
    super(MappingHelper, self).__init__()
    # TODO: Improve this, this is flaky.
    self._tool_path = os.path.dirname(
        os.path.dirname(os.path.realpath(__file__)))

    if not template_path:
      template_path = self._DEFAULT_PATH_TEMPLATE
    # TODO: Check if the project has a YAPF config file and use that instead of
    # falling back to a default one.
    if not formatter_path:
      formatter_path = self._DEFAULT_PATH_FORMATTER

    full_template_path = os.path.join(self._tool_path, template_path)
    self._template_path = full_template_path
    template_loader = jinja2.FileSystemLoader(self._template_path)
    # TODO: Check if autoescape can be set to True due to potential XSS issues.
    self._template_environment = jinja2.Environment(
        autoescape=False, loader=template_loader, trim_blocks=False)

    full_formatter_path = os.path.join(self._tool_path, formatter_path)
    self.formatter = code_formatter.CodeFormatter(full_formatter_path)

  def _RemoveWhitespaceAtEndOfLine(self, template: str) -> str:
    """Removes blanks at the end of lines.

    This is for those parts that are ignored with yapf.

    Args:
      template (str): template to remove end-of-line whitespace from.

    Returns:
      str: template without end-of-line whitespace.
    """
    while template.find('  \n') != -1:
      template = template.replace('  \n', '\n')
    return template

  def _RemoveEscapeError(self, template: str) -> str:
    """Removes the escape error.

    Because jinja template variable is first escaped and then word wrapped,
    the escaped backslash can be split and can result in an EOL.
    The escaped backslash will be placed on the next line.
    This is a workaround and can be removed if yapf supports unicode string
    formatting and it is also changed in the jinja2 template, and only works
    with 8 spaces.

    Args:
      template (str): the resulting template as a python file string

    Returns:
      str: the template without escape (eol) errors
    """
    to_be_replaced = '\\\'\n        u\'\\'
    to_be_replaced_with = '\'\n        u\'\\\\'

    return template.replace(to_be_replaced, to_be_replaced_with)

  def _RemoveYapfComment(self, template: str) -> str:
    """Removes the yapf comment line.

    The line as well as the new line will be removed.
    The yapf comment has to be at the end of the line, or on its own line.

    Args:
      template (str): template to remove yapf comments from.

    Returns:
      str: template with yapf comments removed.
    """
    return template.replace('# yapf: disable\n', '').replace(
        '# yapf: enable\n', '')

  def GenerateClassName(self, scaffolder_name: str) -> str:
    """Generates a class name from the scaffolder name for file generation.

    Args:
      scaffolder_name (str): name of the scaffolder

    Returns:
      str: name of the class
    """
    return scaffolder_name.replace('_', ' ').title().replace(' ', '')

  def RenderTemplate(self, template_filename: str, context: dict) -> str:
    """Renders the template with the context to return a string.

    Args:
      template_filename (str): the name of the template
      context (dict): the context of the template as a dictionary

    Returns:
      str: the rendered template as a string
    """
    template = self._template_environment.get_template(
        template_filename).render(context)
    template = self._RemoveEscapeError(template)

    formatted = self.formatter.Format(template)[0]
    formatted = self._RemoveYapfComment(formatted)
    formatted = self._RemoveWhitespaceAtEndOfLine(formatted)

    return formatted



# !/usr/bin/python
# -*- coding: utf-8 -*-
"""Test class for the file handler."""
import filecmp
import os
import tempfile
import unittest

from l2tscaffolder.lib import file_handler


class FileHandlerTest(unittest.TestCase):
  """Tests for the FileHandler class."""
  name = "__testfile"
  suffix = "py"
  file = '{0:s}.{1:s}'.format(name, suffix)
  maxDiff = None

  def testCreateFolder(self):
    """Tests if the creation of a folder works."""
    with tempfile.TemporaryDirectory() as directory:
      new_path = os.path.join(directory, "newfolder")
      self.assertFalse(os.path.exists(new_path))
      handler = file_handler.FileHandler()
      handler._CreateFolder(new_path)  # pylint: disable=protected-access
      actual = os.path.exists(new_path)
    self.assertTrue(actual)

  def testCreateFilePath(self):
    """Tests if the construction of the folder path works."""
    with tempfile.TemporaryDirectory() as directory:
      new_path = os.path.join(directory, "temp")
      path = file_handler.FileHandler.CreateFilePath(
          new_path, self.name, self.suffix)
      self.assertEqual(path, os.path.join(new_path, self.file))

  def testCreateFile(self):
    """Tests if the creation of a file none existing beforehand works."""
    with tempfile.TemporaryDirectory() as directory:
      handler = file_handler.FileHandler()
      file_path = os.path.join(directory, self.file)
      self.assertFalse(os.path.exists(file_path))
      handler.CreateFile(directory, self.name, self.suffix)
      self.assertTrue(os.path.exists(file_path))

  def testCreateFileIfPathNotExisting(self):
    """Tests creation of non-existing file."""
    with tempfile.TemporaryDirectory() as directory:
      new_path = os.path.join(directory, "temp")
      handler = file_handler.FileHandler()
      file_path = os.path.join(new_path, self.file)
      self.assertFalse(os.path.exists(file_path))
      handler.CreateFile(new_path, self.name, self.suffix)
      self.assertTrue(os.path.exists(file_path))

  def testCreateFileFromPath(self):
    """Tests if the creation of a file none existing beforehand works."""
    handler = file_handler.FileHandler()
    with tempfile.TemporaryDirectory() as directory:
      source = os.path.join(directory, self.file)
      self.assertFalse(os.path.exists(source))
      handler.CreateFileFromPath(source)
      self.assertTrue(os.path.exists(source))

  def testCopyFile(self):
    """Tests if the copying of a file none existing beforehand works."""
    expected_content = "this is test content."

    with tempfile.TemporaryDirectory() as directory:
      source = os.path.join(directory, self.file)
      destination = os.path.join(directory, "copy", self.file)

      with open(source, "a") as f:
        f.write(expected_content)

      handler = file_handler.FileHandler()
      self.assertFalse(os.path.exists(destination))
      handler.CopyFile(source, destination)
      self.assertTrue(os.path.exists(destination))
      self.assertTrue(filecmp.cmp(destination, source))

  def testAddContentIfFileExists(self):
    """Tests if the editing of a file existing works."""
    content = "this is test content. "
    expected = content + content

    with tempfile.TemporaryDirectory() as directory:
      source = os.path.join(directory, self.file)
      with open(source, "a") as f:
        f.write(content)

      handler = file_handler.FileHandler()
      self.assertTrue(os.path.exists(source))
      handler.AddContent(source, content)
      self.assertTrue(os.path.exists(source))

      with open(source, "r") as f:
        actual = f.read()

    self.assertEqual(expected, actual)

  def testAddContentIfFileDoesNotExist(self):
    """Tests if the editing of a file not existing works."""
    content = "this is test content. "
    expected = content

    with tempfile.TemporaryDirectory() as directory:
      source = os.path.join(directory, self.file)
      handler = file_handler.FileHandler()
      self.assertFalse(os.path.exists(source))
      handler.AddContent(source, content)
      self.assertTrue(os.path.exists(source))

      with open(source, "r") as f:
        actual = f.read()

    self.assertEqual(expected, actual)

  def testCreateOrModifyFileWithContentIfFileExists(self):
    """Tests creation or modification of existing file with content."""
    content = "this is test content. "
    expected = content + content

    with tempfile.TemporaryDirectory() as directory:
      source = os.path.join(directory, self.file)
      with open(source, "a") as f:
        f.write(content)

      handler = file_handler.FileHandler()
      self.assertTrue(os.path.exists(source))
      handler.CreateOrModifyFileWithContent(source, content)
      self.assertTrue(os.path.exists(source))

      with open(source, "r") as f:
        actual = f.read()

    self.assertEqual(expected, actual)

  def testAddContentIfFileAndFolderDoesNotExist(self):
    """Tests creation or modification of non-existing file with content."""
    content = "this is test content. "
    expected = content

    with tempfile.TemporaryDirectory() as directory:
      new_path = os.path.join(directory, "newfolder")
      source = os.path.join(new_path, self.file)
      handler = file_handler.FileHandler()
      self.assertFalse(os.path.exists(source))
      handler.CreateOrModifyFileWithContent(source, content)
      self.assertTrue(os.path.exists(source))

      with open(source, "r") as f:
        actual = f.read()

    self.assertEqual(expected, actual)

  def testAddImportToInit(self):
    """Tests adding a line to an init file."""
    test_path = os.path.join(os.path.dirname(
        os.path.dirname(os.path.dirname(__file__))), 'test_data')
    test_file = os.path.join(test_path, 'test_init.py')

    with open(test_file, 'r') as fh:
      test_file_content = fh.read()

    new_import = 'from secret.project.parsers import foobar\n'

    temp_file = tempfile.NamedTemporaryFile()
    with open(temp_file.name, 'w') as fh:
      fh.write(test_file_content)

    handler = file_handler.FileHandler()
    handler.AddImportToInit(temp_file.name, new_import)

    with open(temp_file.name, 'r') as fh:
      content = fh.read()
    os.remove(temp_file.name)

    expected_file_path = os.path.join(test_path, 'test_init_fixed.py')
    with open(expected_file_path, 'r') as fh:
      expected_content = fh.read()

    self.assertEqual(content, expected_content)


if __name__ == '__main__':
  unittest.main()
